from abc import ABC, abstractmethod

from canvas_sdk.effects import Effect
from canvas_sdk.events import EventType
from canvas_sdk.handlers import BaseHandler
from canvas_sdk.handlers.utils import normalize_effects


class Application(BaseHandler, ABC):
    """An embeddable application that can be registered to Canvas."""

    RESPONDS_TO = [
        EventType.Name(EventType.APPLICATION__ON_OPEN),
        EventType.Name(EventType.APPLICATION__ON_CONTEXT_CHANGE),
    ]

    def compute(self) -> list[Effect]:
        """Handle the application events."""
        if self.event.target.id != self.identifier:
            return []
        match self.event.type:
            case EventType.APPLICATION__ON_OPEN:
                return normalize_effects(self.on_open())
            case EventType.APPLICATION__ON_CONTEXT_CHANGE:
                return normalize_effects(self.on_context_change())
            case _:
                return []

    @abstractmethod
    def on_open(self) -> Effect | list[Effect]:
        """Handle the application open event."""
        ...

    def on_context_change(self) -> Effect | list[Effect] | None:
        """Handle the application context change event."""
        return None

    @property
    def identifier(self) -> str:
        """The application identifier."""
        return f"{self.__class__.__module__}:{self.__class__.__qualname__}"


__exports__ = ("Application",)



from canvas_sdk.commands.commands.review.base import _BaseReview
from canvas_sdk.v1.data import ReferralReport


class ReferralReviewCommand(_BaseReview):
    """A class for managing a Referral Review command within a specific note."""

    class Meta:
        key = "referralReview"
        model = ReferralReport


__exports__ = ("ReferralReviewCommand",)



from pydantic_core import InitErrorDetails

from canvas_sdk.commands.base import _BaseCommand as BaseCommand
from canvas_sdk.commands.constants import CodeSystems, Coding


class FamilyHistoryCommand(BaseCommand):
    """A class for managing a Family History command within a specific note."""

    class Meta:
        key = "familyHistory"

    family_history: str | Coding | None = None
    relative: str | None = None
    note: str | None = None

    def _get_error_details(self, method: str) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if (
            isinstance(self.family_history, dict)
            and self.family_history["system"] != CodeSystems.SNOMED
            and self.family_history["system"] != CodeSystems.UNSTRUCTURED
        ):
            message = f"The 'coding.system' field must be '{CodeSystems.SNOMED}' or '{CodeSystems.UNSTRUCTURED}'."
            errors.append(self._create_error_detail("value", message, self.family_history))

        return errors


__exports__ = ("FamilyHistoryCommand",)



from django.db import models
from django.utils import timezone

from canvas_sdk.v1.data.base import IdentifiableModel, TimestampedModel


class EncounterMedium(models.TextChoices):
    """Encounter medium."""

    VOICE = "voice", "Telephone visit"
    VIDEO = "video", "Video visit"
    OFFICE = "office", "Office visit"
    HOME = "home", "Home visit"
    OFFSITE = "offsite", "Other offsite visit"
    LAB = "lab", "Lab visit"


class EncounterState(models.TextChoices):
    """Encounter state."""

    STARTED = "STA", "Started"
    PLANNED = "PLA", "Planned"
    CONCLUDED = "CON", "Concluded"
    CANCELLED = "CAN", "Cancelled"


class Encounter(TimestampedModel, IdentifiableModel):
    """Encounter."""

    class Meta:
        db_table = "canvas_sdk_data_api_encounter_001"

    note = models.OneToOneField("v1.Note", on_delete=models.CASCADE, related_name="encounter")
    medium = models.CharField(choices=EncounterMedium.choices, max_length=20)
    state = models.CharField(max_length=3, choices=EncounterState.choices)
    start_time = models.DateTimeField(default=timezone.now, null=True)
    end_time = models.DateTimeField(default=None, null=True)


__exports__ = (
    "Encounter",
    "EncounterMedium",
    "EncounterState",
)



from .calendar import CalendarFactory, EventFactory
from .claim import (
    ClaimCommentFactory,
    ClaimCoverageFactory,
    ClaimFactory,
    ClaimLabelFactory,
    ClaimProviderFactory,
    ClaimQueueFactory,
    ClaimSubmissionFactory,
)
from .claim_diagnosis_code import ClaimDiagnosisCodeFactory
from .coverage import CoverageFactory
from .facility import FacilityFactory
from .imaging import ImagingOrderFactory, ImagingReportFactory, ImagingReviewFactory
from .lab import (
    LabOrderFactory,
    LabOrderReasonConditionFactory,
    LabOrderReasonFactory,
    LabPartnerFactory,
    LabPartnerTestFactory,
    LabReportFactory,
    LabReviewFactory,
    LabTestFactory,
    LabValueCodingFactory,
    LabValueFactory,
)
from .medication_history import (
    MedicationHistoryMedicationCodingFactory,
    MedicationHistoryMedicationFactory,
    MedicationHistoryResponseFactory,
)
from .note import NoteFactory, NoteStateChangeEventFactory, NoteTypeFactory
from .organization import (
    OrganizationAddressFactory,
    OrganizationContactPointFactory,
    OrganizationFactory,
)
from .patient import PatientAddressFactory, PatientFacilityAddressFactory, PatientFactory
from .practicelocation import (
    PracticeLocationAddressFactory,
    PracticeLocationContactPointFactory,
    PracticeLocationFactory,
    PracticeLocationSettingFactory,
)
from .protocol_current import ProtocolCurrentFactory
from .referral import ReferralFactory, ReferralReportFactory, ReferralReviewFactory
from .staff import (
    StaffAddressFactory,
    StaffContactPointFactory,
    StaffFactory,
    StaffLicenseFactory,
    StaffPhotoFactory,
    StaffRoleFactory,
)
from .task import (
    TaskCommentFactory,
    TaskFactory,
    TaskLabelFactory,
    TaskMetadataFactory,
    TaskTaskLabelFactory,
)
from .uncategorized_clinical_document import (
    UncategorizedClinicalDocumentFactory,
    UncategorizedClinicalDocumentReviewFactory,
)
from .user import CanvasUserFactory

__all__ = (
    "CalendarFactory",
    "CanvasUserFactory",
    "ClaimFactory",
    "ClaimCommentFactory",
    "ClaimCoverageFactory",
    "ClaimDiagnosisCodeFactory",
    "ClaimLabelFactory",
    "ClaimProviderFactory",
    "ClaimQueueFactory",
    "ClaimSubmissionFactory",
    "CoverageFactory",
    "EventFactory",
    "FacilityFactory",
    "ImagingOrderFactory",
    "ImagingReportFactory",
    "ImagingReviewFactory",
    "LabOrderFactory",
    "LabOrderReasonConditionFactory",
    "LabOrderReasonFactory",
    "LabPartnerFactory",
    "LabPartnerTestFactory",
    "LabReportFactory",
    "LabReviewFactory",
    "LabTestFactory",
    "LabValueCodingFactory",
    "LabValueFactory",
    "MedicationHistoryMedicationFactory",
    "MedicationHistoryMedicationCodingFactory",
    "MedicationHistoryResponseFactory",
    "NoteFactory",
    "NoteStateChangeEventFactory",
    "NoteTypeFactory",
    "OrganizationAddressFactory",
    "OrganizationContactPointFactory",
    "OrganizationFactory",
    "PatientAddressFactory",
    "PatientFacilityAddressFactory",
    "PatientFactory",
    "PracticeLocationFactory",
    "PracticeLocationAddressFactory",
    "PracticeLocationContactPointFactory",
    "PracticeLocationSettingFactory",
    "ProtocolCurrentFactory",
    "ReferralFactory",
    "ReferralReportFactory",
    "ReferralReviewFactory",
    "StaffFactory",
    "StaffPhotoFactory",
    "StaffRoleFactory",
    "StaffLicenseFactory",
    "StaffContactPointFactory",
    "StaffAddressFactory",
    "TaskCommentFactory",
    "TaskFactory",
    "TaskLabelFactory",
    "TaskMetadataFactory",
    "TaskTaskLabelFactory",
    "UncategorizedClinicalDocumentFactory",
    "UncategorizedClinicalDocumentReviewFactory",
)



from __future__ import annotations

import json
from http import HTTPStatus

from requests import exceptions

from canvas_sdk.clients.llms.libraries.llm_api import LlmApi
from canvas_sdk.clients.llms.structures.llm_response import LlmResponse
from canvas_sdk.clients.llms.structures.llm_tokens import LlmTokens


class LlmOpenai(LlmApi):
    """OpenAI LLM API client.

    Implements the LlmBase interface for OpenAI's API.
    """

    def to_dict(self) -> dict:
        """Convert prompts and add the necessary information to OpenAI API request format.

        Returns:
            Dictionary formatted for OpenAI API with instructions and input messages.
        """
        roles = {
            self.ROLE_SYSTEM: "developer",
            self.ROLE_USER: "user",
            self.ROLE_MODEL: "assistant",
        }
        messages: list[dict] = [
            {
                "role": roles[prompt.role],
                "content": [
                    {
                        "type": "input_text" if prompt.role == self.ROLE_USER else "output_text",
                        "text": "\n".join(prompt.text),
                    }
                ],
            }
            for prompt in self.prompts
            if prompt.role != self.ROLE_SYSTEM
        ]

        system_prompt = "\n".join(
            ["\n".join(prompt.text) for prompt in self.prompts if prompt.role == self.ROLE_SYSTEM]
        )
        return self.settings.to_dict() | {
            "instructions": system_prompt,
            "input": messages,
        }

    @classmethod
    def _api_base_url(cls) -> str:
        return "https://us.api.openai.com"

    def request(self) -> LlmResponse:
        """Make a request to the OpenAI API.

        Returns:
            Response containing status code, generated text, and token usage.
        """
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.settings.api_key}",
        }
        data = json.dumps(self.to_dict())

        tokens = LlmTokens(prompt=0, generated=0)
        try:
            request = self.http.post("/v1/responses", headers=headers, data=data)
            code = request.status_code
            response = request.text
            if code == HTTPStatus.OK.value:
                content = json.loads(request.text)
                response = ""
                for output in content.get("output", [{}]):
                    if output.get("type", "") == "message":
                        response += output.get("content", [{}])[0].get("text", "")
                usage = content.get("usage", {})
                tokens = LlmTokens(
                    prompt=usage.get("input_tokens") or 0,
                    generated=usage.get("output_tokens") or 0,
                )
        except exceptions.RequestException as e:
            code = HTTPStatus.BAD_REQUEST
            response = f"Request failed: {e}"
            if message := getattr(e, "response", None):
                code = message.status_code
                response = message.text

        return LlmResponse(
            code=HTTPStatus(code),
            response=response,
            tokens=tokens,
        )


__exports__ = ("LlmOpenai",)



from canvas_sdk.effects.payment.base import ClaimAllocation, LineItemTransaction, PaymentMethod
from canvas_sdk.effects.payment.post_claim_payment import (
    PostClaimPayment,
)

__all__ = __exports__ = (
    "PostClaimPayment",
    "ClaimAllocation",
    "LineItemTransaction",
    "PaymentMethod",
)



import json
from http import HTTPStatus

from requests import exceptions

from canvas_sdk.clients.llms.libraries.llm_api import LlmApi
from canvas_sdk.clients.llms.structures.llm_response import LlmResponse
from canvas_sdk.clients.llms.structures.llm_tokens import LlmTokens


class LlmGoogle(LlmApi):
    """Google Gemini LLM API client.

    Implements the LlmBase interface for Google's Generative Language API.
    """

    def to_dict(self) -> dict:
        """Convert prompts and add the necessary information to Google API request format.

        Returns:
            Dictionary formatted for Google API with contents array.
        """
        contents: list[dict] = []
        roles = {
            self.ROLE_SYSTEM: "user",
            self.ROLE_USER: "user",
            self.ROLE_MODEL: "model",
        }
        for prompt in self.prompts:
            role = roles[prompt.role]
            part = {"text": "\n".join(prompt.text)}
            # contiguous parts for the same role are merged
            if contents and contents[-1]["role"] == role:
                contents[-1]["parts"].append(part)
            else:
                contents.append({"role": role, "parts": [part]})

        return self.settings.to_dict() | {
            "contents": contents,
        }

    @classmethod
    def _api_base_url(cls) -> str:
        return "https://generativelanguage.googleapis.com"

    def request(self) -> LlmResponse:
        """Make a request to the Google Gemini API.

        Returns:
            Response containing status code, generated text, and token usage.
        """
        headers = {"Content-Type": "application/json"}
        data = json.dumps(self.to_dict())

        tokens = LlmTokens(prompt=0, generated=0)
        try:
            request = self.http.post(
                f"/v1beta/{self.settings.model}:generateContent?key={self.settings.api_key}",
                headers=headers,
                data=data,
            )
            code = request.status_code
            response = request.text
            if code == HTTPStatus.OK.value:
                content = json.loads(request.text)
                response = (
                    content.get("candidates", [{}])[0]
                    .get("content", {})
                    .get("parts", [{}])[0]
                    .get("text", "")
                )
                usage = content.get("usageMetadata", {})
                tokens = LlmTokens(
                    prompt=usage.get("promptTokenCount") or 0,
                    generated=(usage.get("candidatesTokenCount") or 0)
                    + (usage.get("thoughtsTokenCount") or 0),
                )
        except exceptions.RequestException as e:
            code = HTTPStatus.BAD_REQUEST
            response = f"Request failed: {e}"
            if message := getattr(e, "response", None):
                code = message.status_code
                response = message.text

        return LlmResponse(
            code=HTTPStatus(code),
            response=response,
            tokens=tokens,
        )


__exports__ = ("LlmGoogle",)



# To run the tests, use the command `pytest` in the terminal or uv run pytest.
# Each test is wrapped inside a transaction that is rolled back at the end of the test.
# If you want to modify which files are used for testing, check the [tool.pytest.ini_options] section in pyproject.toml.
# For more information on testing Canvas plugins, see: https://docs.canvasmedical.com/sdk/testing-utils/

from unittest.mock import Mock

import pytest
from canvas_sdk.effects import EffectType
from canvas_sdk.events import EventType
from canvas_sdk.test_utils.factories import PatientFactory
from canvas_sdk.v1.data.discount import Discount

from {{ cookiecutter.__package_name }}.protocols.my_protocol import Protocol


# Test the protocol's compute method with mocked event data
def test_protocol_responds_to_assess_command() -> None:
    """Test that the protocol responds correctly to ASSESS_COMMAND__CONDITION_SELECTED events."""
    # Create a mock event with the expected structure
    mock_event = Mock()
    mock_event.type = EventType.ASSESS_COMMAND__CONDITION_SELECTED
    mock_event.context = {
        "note": {
            "uuid": "test-note-uuid-123",
        }
    }

    # Instantiate the protocol with the mock event
    protocol = Protocol(event=mock_event)

    # Call compute and get the effects
    effects = protocol.compute()

    # Assert that effects were returned
    assert len(effects) == 1

    # Assert the effect has the correct type
    assert effects[0].type == EffectType.LOG

    # Assert the effect contains expected data
    assert "test-note-uuid-123" in effects[0].payload
    assert protocol.NARRATIVE_STRING in effects[0].payload


# Test that the protocol class has the correct event type configured
def test_protocol_event_configuration() -> None:
    """Test that the protocol is configured to respond to the correct event type."""
    assert EventType.Name(EventType.ASSESS_COMMAND__CONDITION_SELECTED) == Protocol.RESPONDS_TO


# Example: You can use a factory to create a patient instance for testing purposes.
def test_factory_example() -> None:
    """Test that a patient can be created using the PatientFactory."""
    patient = PatientFactory.create()
    assert patient.id is not None


# Example: If a factory is not available, you can create an instance manually with the data model directly.
def test_model_example() -> None:
    """Test that a Discount instance can be created."""
    Discount.objects.create(name="10%", adjustment_group="30", adjustment_code="CO", discount=0.10)
    assert Discount.objects.first().pk is not None



from ..value_set import ValueSet


class RetinalOrDilatedEyeExam(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a physical exam where a retinal or dilated eye exam was performed.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam during which a retinal or dilated eye exam occurred.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS131v10
    """

    VALUE_SET_NAME = "Retinal or Dilated Eye Exam"
    OID = "2.16.840.1.113883.3.464.1003.115.12.1088"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "6615001",  # Electroretinography (procedure)
        "252779009",  # Single bright white flash electroretinography (procedure)
        "252780007",  # Dark adapted single bright flash electroretinography (procedure)
        "252781006",  # Pre-dark-adapted single bright flash electroretinography (procedure)
        "252782004",  # Photopic electroretinography (procedure)
        "252783009",  # Scotopic rod electroretinography (procedure)
        "252784003",  # Flicker electroretinography (procedure)
        "252788000",  # Chromatic electroretinography (procedure)
        "252789008",  # Early receptor potential electroretinography (procedure)
        "252790004",  # Focal electroretinography (procedure)
        "274795007",  # Examination of optic disc (procedure)
        "274798009",  # Examination of retina (procedure)
        "308110009",  # Direct fundoscopy following mydriatic (procedure)
        "314971001",  # Camera fundoscopy (procedure)
        "314972008",  # Indirect fundoscopy following mydriatic (procedure)
        "410451008",  # Indirect ophthalmoscopy (procedure)
        "410452001",  # Monocular indirect ophthalmoscopy (procedure)
        "410453006",  # Binocular indirect ophthalmoscopy (procedure)
        "410455004",  # Slit lamp fundus examination (procedure)
        "420213007",  # Multifocal electroretinography (procedure)
        "425816006",  # Ultrasonic evaluation of retina (procedure)
        "427478009",  # Evaluation of retina (procedure)
        "722161008",  # Diabetic retinal eye exam (procedure)
    }


class BestCorrectedVisualAcuityExamUsingSnellenChart(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a physical exam for visual acuity exams using a Snellen chart.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam for visual acuity using a Snellen chart.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS133v10
    """

    VALUE_SET_NAME = "Best Corrected Visual Acuity Exam Using Snellen Chart"
    OID = "2.16.840.1.113883.3.526.3.1560"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "79880-1",  # Visual acuity best corrected Right eye by Snellen eye chart
        "79881-9",  # Visual acuity best corrected Left eye by Snellen eye chart
    }


class HeartRate(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a physical exam measuring heart rate.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam for obtaining heart rate.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS144v10, CMS145v10
    """

    VALUE_SET_NAME = "Heart Rate"
    OID = "2.16.840.1.113883.3.526.3.1176"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "68999-2",  # Heart rate --supine
        "69000-8",  # Heart rate --sitting
        "69001-6",  # Heart rate --standing
        "8867-4",  # Heart rate
    }


class BmiPercentile(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a physical exam where a body mass index (BMI) percentile is calculated.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam with a BMI percentile measurement.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS155v10
    """

    VALUE_SET_NAME = "BMI percentile"
    OID = "2.16.840.1.113883.3.464.1003.121.12.1012"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "59574-4",  # Body mass index (BMI) [Percentile]
        "59575-1",  # Body mass index (BMI) [Percentile] Per age
        "59576-9",  # Body mass index (BMI) [Percentile] Per age and sex
    }


class Height(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a physical exam with measured height.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam with a patient height measurement.

    **Exclusion Criteria:** Excludes concepts that represent patient reported height, a percentile for height or an estimated height.

    ** Used in:** CMS155v10
    """

    VALUE_SET_NAME = "Height"
    OID = "2.16.840.1.113883.3.464.1003.121.12.1014"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "3137-7",  # Body height Measured
        "8302-2",  # Body height
        "8306-3",  # Body height --lying
        "8307-1",  # Body height --preoperative
        "8308-9",  # Body height --standing
    }


class Weight(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a physical exam with measured weight.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam with a patient weight measurement.

    **Exclusion Criteria:** Excludes concepts that represent an ideal body weight, estimated body weight or a body fat measurement.

    ** Used in:** CMS155v10
    """

    VALUE_SET_NAME = "Weight"
    OID = "2.16.840.1.113883.3.464.1003.121.12.1015"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "18833-4",  # First Body weight
        "29463-7",  # Body weight
        "3141-9",  # Body weight Measured
        "3142-7",  # Body weight Stated
        "8341-0",  # Dry body weight Measured
        "8349-3",  # Body weight Measured --preoperative
        "8350-1",  # Body weight Measured --with clothes
        "8351-9",  # Body weight Measured --without clothes
    }


class BmiRatio(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a physical exam where a body mass index (BMI) ratio is calculated.

    **Data Element Scope:** This value set may use a model element related to Physical Exam.

    **Inclusion Criteria:** Includes concepts that represent a physical exam with a body mass index (BMI) ratio measurement.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS249v4
    """

    VALUE_SET_NAME = "BMI Ratio"
    OID = "2.16.840.1.113883.3.600.1.1490"
    DEFINITION_VERSION = "20140502"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "39156-5",  # Body mass index (BMI) [Ratio]
    }


__exports__ = (
    "BestCorrectedVisualAcuityExamUsingSnellenChart",
    "BmiPercentile",
    "BmiRatio",
    "HeartRate",
    "Height",
    "RetinalOrDilatedEyeExam",
    "Weight",
)



import json
from datetime import datetime
from enum import StrEnum
from typing import Any
from uuid import UUID

from pydantic_core import InitErrorDetails

from canvas_generated.messages.effects_pb2 import Effect
from canvas_sdk.effects import EffectType, _BaseEffect


class EventRecurrence(StrEnum):
    """Calendar event recurrence."""

    Daily = "DAILY"
    Weekly = "WEEKLY"


class DaysOfWeek(StrEnum):
    """Days of the week."""

    Monday = "MO"
    Tuesday = "TU"
    Wednesday = "WE"
    Thursday = "TH"
    Friday = "FR"
    Saturday = "SA"
    Sunday = "SU"


def get_recurrence_string(
    frequency: EventRecurrence | None, interval: int | None, days_of_week: list[DaysOfWeek] | None
) -> str:
    """Generate the recurrence string for an event."""
    parts = [
        f"FREQ={frequency}" if frequency else "",
        f"INTERVAL={interval}" if interval else "",
        f"BYDAY={','.join(list(map(str, days_of_week)))}" if days_of_week else "",
    ]

    return ";".join(filter(bool, parts))


class Event(_BaseEffect):
    """Effect to create a Calendar event."""

    calendar_id: str | UUID | None = None
    event_id: str | UUID | None = None
    title: str | None = None
    starts_at: datetime | None = None
    ends_at: datetime | None = None
    recurrence_frequency: EventRecurrence | None = None
    recurrence_interval: int | None = None
    recurrence_days: list[DaysOfWeek] | None = None
    recurrence_ends_at: datetime | None = None
    allowed_note_types: list[str] | None = None

    @property
    def values(self) -> dict[str, Any]:
        """The event's values."""
        return {
            "event_id": self.event_id,
            "calendar_id": self.calendar_id,
            "title": self.title,
            "starts_at": self.starts_at.isoformat() if self.starts_at else None,
            "ends_at": self.ends_at.isoformat() if self.ends_at else None,
            "recurrence": get_recurrence_string(
                self.recurrence_frequency, self.recurrence_interval, self.recurrence_days
            ),
            "recurrence_ends_at": self.recurrence_ends_at.isoformat()
            if self.recurrence_ends_at
            else None,
            "allowed_note_types": self.allowed_note_types,
        }

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        # calendar_id is required for create
        if method == "create" and not self.calendar_id:
            errors.append(
                self._create_error_detail(
                    "missing",
                    "Field 'calendar_id' is required to create an event.",
                    None,
                )
            )

        if method in ("update", "delete") and not self.event_id:
            errors.append(
                self._create_error_detail(
                    "missing",
                    f"Field 'event_id' is required to {method} an event.",
                    None,
                )
            )

        if method in (
            "create",
            "update",
        ):
            if not self.title:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        f"Field 'title' is required to {method} an event.",
                        None,
                    )
                )

            if not self.starts_at:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        f"Field 'starts_at' is required to {method} an event.",
                        None,
                    )
                )

            if not self.ends_at:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        f"Field 'ends_at' is required to {method} an event.",
                        None,
                    )
                )

        return errors

    def create(self) -> Effect:
        """Send a CREATE effect for the calendar event."""
        self._validate_before_effect("create")

        return Effect(
            type=EffectType.CALENDAR__EVENT__CREATE,
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )

    def update(self) -> Effect:
        """Send an UPDATE effect for the calendar event."""
        self._validate_before_effect("update")

        return Effect(
            type=EffectType.CALENDAR__EVENT__UPDATE,
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )

    def delete(self) -> Effect:
        """Send a DELETE effect for the calendar event."""
        self._validate_before_effect("delete")

        return Effect(
            type=EffectType.CALENDAR__EVENT__DELETE,
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )


__exports__ = ("Event", "EventRecurrence", "DaysOfWeek")



from enum import Enum
from typing import Any

from pydantic import BaseModel, ConfigDict

from canvas_sdk.effects.base import EffectType, _BaseEffect


class Recommendation(BaseModel):
    """
    A Recommendation for a Protocol Card.
    """

    model_config = ConfigDict(strict=True, validate_assignment=True)

    title: str = ""
    button: str = ""
    href: str | None = None
    command: str | None = None
    context: dict | None = None

    @property
    def values(self) -> dict:
        """The ProtocolCard recommendation's values."""
        return {
            "title": self.title,
            "button": self.button,
            "href": self.href,
            "command": {"type": self.command} if self.command else {},
            "context": self.context or {},
        }


class ProtocolCard(_BaseEffect):
    """
    An Effect that will result in a protocol card in Canvas.
    """

    class Status(Enum):
        DUE = "due"
        SATISFIED = "satisfied"
        NOT_APPLICABLE = "not_applicable"
        PENDING = "pending"
        NOT_RELEVANT = "not_relevant"

    class Meta:
        effect_type = EffectType.ADD_OR_UPDATE_PROTOCOL_CARD
        apply_required_fields = ("patient_id|patient_filter", "key")

    patient_id: str | None = None
    key: str | None = None
    title: str = ""
    narrative: str = ""
    recommendations: list[Recommendation] = []
    status: Status = Status.DUE
    feedback_enabled: bool = False
    due_in: int = -1
    can_be_snoozed: bool = False

    @property
    def values(self) -> dict[str, Any]:
        """The ProtocolCard's values."""
        return {
            "title": self.title,
            "narrative": self.narrative,
            "recommendations": [
                rec.values | {"key": i} for i, rec in enumerate(self.recommendations)
            ],
            "status": self.status.value,
            "feedback_enabled": self.feedback_enabled,
            "due_in": self.due_in,
            "can_be_snoozed": self.can_be_snoozed,
        }

    @property
    def effect_payload(self) -> dict[str, Any]:
        """The payload of the effect."""
        return {
            "patient": self.patient_id,
            "patient_filter": self.patient_filter,
            "key": self.key,
            "data": self.values,
        }

    def add_recommendation(
        self,
        title: str = "",
        button: str = "",
        href: str | None = None,
        command: str | None = None,
        context: dict | None = None,
    ) -> None:
        """Adds a recommendation to the protocol card's list of recommendations."""
        recommendation = Recommendation(
            title=title, button=button, href=href, command=command, context=context
        )
        self.recommendations.append(recommendation)


__exports__ = (
    "Recommendation",
    "ProtocolCard",
)



from canvas_sdk.effects import EffectType
from canvas_sdk.effects.form import BaseCreateFormEffect, FormField, InputType  # noqa


class PatientMetadataCreateFormEffect(BaseCreateFormEffect):
    """An Effect that will create a form."""

    class Meta:
        effect_type = EffectType.PATIENT_METADATA__CREATE_ADDITIONAL_FIELDS


__exports__ = ("PatientMetadataCreateFormEffect", "FormField", "InputType")



from enum import Enum
from typing import Any

from pydantic import Field
from pydantic_core import InitErrorDetails

from canvas_sdk.effects.base import EffectType, _BaseEffect


class PanelConfiguration(_BaseEffect):
    """
    An Effect that will decide which buttons appear on the panel section in Canvas.
    """

    class Meta:
        effect_type = EffectType.SHOW_PANEL_SECTIONS

    class Page(Enum):
        GLOBAL = "global"
        PATIENT = "patient"

    class PanelGlobalSection(Enum):
        APPOINTMENT = "appointment"
        CHANGE_REQUEST = "changeRequest"
        IMAGING_REPORT = "imagingReport"
        INPATIENT_STAY = "inpatientStay"
        LAB_REPORT = "labReport"
        MESSAGE = "message"
        OUTSTANDING_REFERRAL = "outstandingReferral"
        PRESCRIPTION_ALERT = "prescriptionAlert"
        RECALL_APPOINTMENT = "recallAppointment"
        REFERRAL_REPORT = "referralReport"
        REFILL_REQUEST = "refillRequest"
        TASK = "task"
        UNCATEGORIZED_DOCUMENT = "uncategorizedDocument"

    class PanelPatientSection(Enum):
        CHANGE_REQUEST = "changeRequest"
        COMMAND = "command"
        IMAGING_REPORT = "imagingReport"
        INPATIENT_STAY = "inpatientStay"
        LAB_REPORT = "labReport"
        PRESCRIPTION_ALERT = "prescriptionAlert"
        REFERRAL_REPORT = "referralReport"
        REFILL_REQUEST = "refillRequest"
        TASK = "task"
        UNCATEGORIZED_DOCUMENT = "uncategorizedDocument"

    sections: list[PanelPatientSection] | list[PanelGlobalSection] = Field(min_length=1)
    page: Page

    @property
    def values(self) -> dict[str, Any]:
        """The PanelConfiguration's values."""
        return {"sections": [s.value for s in self.sections]}

    @property
    def effect_payload(self) -> dict[str, Any]:
        """The payload of the effect."""
        return {"data": self.values}

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if self.page == PanelConfiguration.Page.GLOBAL and not all(
            isinstance(s, PanelConfiguration.PanelGlobalSection) for s in self.sections
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    "All sections must be of type PanelGlobalSection",
                    self.sections,
                )
            )

        if self.page == PanelConfiguration.Page.PATIENT and not all(
            isinstance(s, PanelConfiguration.PanelPatientSection) for s in self.sections
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    "All sections must be of type PanelPatientSection",
                    self.sections,
                )
            )

        return errors


__exports__ = ("PanelConfiguration",)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, cast

import arrow

from canvas_sdk.events import EventType
from canvas_sdk.protocols.base import BaseProtocol
from canvas_sdk.protocols.timeframe import Timeframe
from canvas_sdk.v1.data import Patient
from canvas_sdk.v1.data.condition import Condition
from canvas_sdk.v1.data.lab import LabOrder, LabReport
from canvas_sdk.v1.data.medication import Medication
from canvas_sdk.v1.data.protocol_override import ProtocolOverride

if TYPE_CHECKING:
    from django.db.models import Model


class ClinicalQualityMeasure(BaseProtocol):
    """
    The class that ClinicalQualityMeasure protocols inherit from.
    """

    class Meta:
        title: str = ""
        identifiers: list[str] = []
        description: str = ""
        information: str = ""
        references: list[str] = []
        source_attributes: dict[str, str]
        types: list[str] = []
        authors: list[str] = []
        show_in_chart: bool = True
        show_in_population: bool = True
        can_be_snoozed: bool = True
        is_abstract: bool = False
        is_predictive: bool = False

    def __init__(self, *args: Any, **kwargs: Any):
        self._patient_id: str | None = None
        self.now = arrow.utcnow()
        super().__init__(*args, **kwargs)

    @classmethod
    def _meta(cls) -> dict[str, Any]:
        """
        Meta properties of the protocol in dictionary form.
        """
        base_meta = {
            k: v for k, v in ClinicalQualityMeasure.Meta.__dict__.items() if not k.startswith("__")
        }
        class_meta = {k: v for k, v in cls.Meta.__dict__.items() if not k.startswith("__")}

        return base_meta | class_meta

    @classmethod
    def protocol_key(cls) -> str:
        """
        External key used to identify the protocol.
        """
        return cls.__name__

    @staticmethod
    def relative_float(value: str) -> float:
        """Relative float method (better explanation needed here)."""
        try:
            sigma = 0.0
            if value[0] == "<" and value[1] != "=":
                sigma = -1e-6
            elif value[0] == ">" and value[1] != "=":
                sigma = +1e-6
            return float(value.strip("<≤=≥>")) + sigma
        except (ValueError, IndexError):
            return 0

    @property
    def timeframe(self) -> Timeframe:
        """The default Timeframe (self.timeframe) for all protocols.
        This defaults to have a start of 1 year ago and an end time of the current time.
        Plugin authors can override this if a different timeframe is desired.
        """
        end = self.now
        return Timeframe(start=end.shift(years=-1), end=end)

    # TODO: This approach should be considered against the alternative of just including the patient
    #  ID in the event context, given that so many events will be patient-centric.
    def patient_id_from_target(self) -> str:
        """
        Get and return the patient ID from an event target.

        This method will attempt to obtain the patient ID from the event target for supported event
        types. It stores the patient ID on a member variable so that it can be referenced without
        incurring more SQL queries.
        """

        def patient_id(model: type[Model]) -> str:
            if model == Patient:
                return self.event.target.id
            else:
                return cast(
                    str,
                    model._default_manager.select_related("patient")
                    .values_list("patient__id")
                    .get(id=self.event.target)[0],
                )

        models = {
            EventType.CONDITION_ASSESSED: Condition,
            EventType.CONDITION_CREATED: Condition,
            EventType.CONDITION_RESOLVED: Condition,
            EventType.CONDITION_UPDATED: Condition,
            EventType.LAB_ORDER_CREATED: LabOrder,
            EventType.LAB_ORDER_UPDATED: LabOrder,
            EventType.LAB_REPORT_CREATED: LabReport,
            EventType.LAB_REPORT_UPDATED: LabReport,
            EventType.MEDICATION_LIST_ITEM_CREATED: Medication,
            EventType.MEDICATION_LIST_ITEM_UPDATED: Medication,
            EventType.PATIENT_CREATED: Patient,
            EventType.PATIENT_UPDATED: Patient,
            EventType.PROTOCOL_OVERRIDE_CREATED: ProtocolOverride,
            EventType.PROTOCOL_OVERRIDE_DELETED: ProtocolOverride,
            EventType.PROTOCOL_OVERRIDE_UPDATED: ProtocolOverride,
        }

        if not self._patient_id:
            try:
                self._patient_id = patient_id(models[self.event.type])
            except KeyError as error:
                raise ValueError(
                    f"Event type {self.event.type} not supported by 'patient_id_from_event'"
                ) from error

        return self._patient_id


__exports__ = ("ClinicalQualityMeasure",)



from typing import Any

from pydantic import Field
from pydantic_core import InitErrorDetails

from canvas_generated.messages.effects_pb2 import EffectType
from canvas_sdk.effects.base import _BaseEffect
from canvas_sdk.v1.data import Patient, Staff


class GenerateFullChartPDFEffect(_BaseEffect):
    """
    An Effect that will generate a full chart PDF for a patient.

    Generated PDF will appear in task list assigned to requestor with "chart pdf" label within 10 minutes.
    """

    class Meta:
        effect_type = EffectType.GENERATE_FULL_CHART_PDF

    patient_id: str = Field(min_length=1)
    requestor_staff_id: str = Field(min_length=1)

    @property
    def values(self) -> dict[str, Any]:
        """The GenerateFullChartPDFEffect's values."""
        return {
            "patient_id": self.patient_id,
            "requestor_staff_id": self.requestor_staff_id,
        }

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Patient with ID {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        if not Staff.objects.filter(id=self.requestor_staff_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Requestor staff with ID {self.requestor_staff_id} does not exist.",
                    self.requestor_staff_id,
                )
            )

        return errors


__exports__ = ("GenerateFullChartPDFEffect",)



from typing import Any

from canvas_sdk.commands.constants import Coding
from canvas_sdk.effects.base import EffectType, _BaseEffect


class AddBillingLineItem(_BaseEffect):
    """
    An Effect that will result in a billing line item in a note footer.
    """

    class Meta:
        effect_type = EffectType.ADD_BILLING_LINE_ITEM
        apply_required_fields = ("note_id", "cpt")

    note_id: str | None = None
    cpt: str | None = None
    units: int | None = 1
    assessment_ids: list[str] = []
    modifiers: list[Coding] = []

    @property
    def values(self) -> dict[str, Any]:
        """The BillingLineItem's values."""
        return {
            "cpt": self.cpt,
            "units": self.units,
            "assessment_ids": self.assessment_ids,
            "modifiers": self.modifiers,
        }

    @property
    def effect_payload(self) -> dict[str, Any]:
        """The payload of the effect."""
        return {
            "note_id": self.note_id,
            "data": self.values,
        }


__exports__ = ("AddBillingLineItem",)



from pydantic import Field

from canvas_sdk.commands.commands.refill import RefillCommand


class AdjustPrescriptionCommand(RefillCommand):
    """A class for managing Adjust Prescription command within a specific note."""

    class Meta:
        key = "adjustPrescription"

    new_fdb_code: str | None = Field(
        default=None, json_schema_extra={"commands_api_name": "change_medication_to"}
    )

    def _has_fdb_code(self) -> bool:
        """Check if new_fdb_code is provided and non-empty."""
        return self.new_fdb_code is not None and self.new_fdb_code.strip() != ""


__exports__ = ("AdjustPrescriptionCommand",)



from canvas_sdk.effects.calendar.calendar import Calendar, CalendarType
from canvas_sdk.effects.calendar.event import (
    DaysOfWeek,
    Event,
    EventRecurrence,
)

__all__ = __exports__ = (
    "Calendar",
    "CalendarType",
    "Event",
    "EventRecurrence",
    "DaysOfWeek",
)



from enum import StrEnum
from typing import Any, Self

from pydantic import model_validator
from pydantic_core import InitErrorDetails

from canvas_sdk.effects import EffectType, _BaseEffect


class PortalWidget(_BaseEffect):
    """An Effect that will launch a modal."""

    class Meta:
        effect_type = EffectType.PORTAL_WIDGET

    class Component(StrEnum):
        APPOINTMENTS = "appointments"
        MESSAGING = "messaging"

    class Size(StrEnum):
        EXPANDED = "expanded"
        MEDIUM = "medium"
        COMPACT = "compact"

    url: str | None = None
    content: str | None = None
    component: Component | None = None
    priority: int = 100
    size: Size = Size.EXPANDED

    @property
    def values(self) -> dict[str, Any]:
        """The PortalWidget values."""
        return {
            "url": self.url,
            "content": self.content,
            "component": self.component,
            "size": self.size,
            "priority": self.priority,
        }

    @model_validator(mode="after")
    def check_mutually_exclusive_fields(self) -> Self:
        """Check that content and component are mutually exclusive."""
        if self.content is not None and self.component is not None:
            raise ValueError("'content' and 'component' are mutually exclusive")

        if self.content is not None and self.url is not None:
            raise ValueError("'content' and 'url' are mutually exclusive")

        if self.url is not None and self.component is not None:
            raise ValueError("'url' and 'component' are mutually exclusive")

        return self

    @model_validator(mode="after")
    def check_exclusive_fields(self) -> Self:
        """Check that at least one of mutually exclusive field is set."""
        if self.content is None and self.component is None and self.url is None:
            raise ValueError("one of 'content', 'component', 'url' is required")

        return self

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if self.component is not None and self.size != PortalWidget.Size.EXPANDED:
            errors.append(
                self._create_error_detail(
                    "value",
                    "Component only supports 'expanded' size",
                    self.size,
                )
            )

        return errors


__exports__ = ("PortalWidget",)



from collections.abc import Generator
from contextlib import contextmanager
from pathlib import Path

import django.db
import typer
from django.core.management import call_command

import settings
from plugin_runner.plugin_runner import main as run_server


@contextmanager
def _patch_default_db_connection() -> Generator[None, None, None]:
    """
    Temporarily patch the 'default' Django DB connection to allow db writes.
    """
    original_default = django.db.connections["default"]
    try:
        temp_handler = django.db.utils.ConnectionHandler(
            {"default": settings.SQLITE_WRITE_MODE_DATABASE}
        )
        django.db.connections["default"] = temp_handler["default"]
        yield
    finally:
        django.db.connections["default"] = original_default


def _run_db_seed_file(path: Path) -> None:
    """Run the database setup file to initialize the database."""
    code = path.read_text()
    exec_globals = {"__name__": "__main__"}
    exec(code, exec_globals)


def _reset_db(seed_file: Path | None = None) -> None:
    """Reset the database."""
    settings.SQLITE_DB_PATH.unlink(missing_ok=True)

    with _patch_default_db_connection():
        call_command("migrate", run_syncdb=True, verbosity=0)
        if seed_file:
            _run_db_seed_file(seed_file)


def run_plugin(
    plugin_directory: str = typer.Argument(..., help="Path to the plugin directory to run."),
    db_seed_file: Path | None = typer.Option(
        help="Path to the database seed file to use.",
        default=None,
    ),
    reset_db: bool = typer.Option(
        help="Reset the database before running the plugin.", default=False
    ),
) -> None:
    """
    Run the specified plugin for local development.
    """
    _run_plugins([plugin_directory], db_seed_file=db_seed_file, reset_db=reset_db)


def run_plugins(
    plugin_directories: list[str],
    db_seed_file: Path | None = typer.Option(
        help="Path to the database seed file to use.",
        default=None,
    ),
    reset_db: bool = typer.Option(
        help="Reset the database before running the plugin(s).", default=False
    ),
) -> None:
    """
    Run the specified plugins for local development.
    """
    _run_plugins(plugin_directories, db_seed_file=db_seed_file, reset_db=reset_db)


def _run_plugins(
    plugin_directories: list[str], db_seed_file: Path | None = None, reset_db: bool = False
) -> None:
    """
    Run the specified plugins for local development.
    """
    if db_seed_file:
        if not db_seed_file.exists():
            raise typer.BadParameter(f"Database setup file not found: {db_seed_file.resolve()}")
        if not db_seed_file.is_file():
            raise typer.BadParameter(
                f"Database setup file is not a regular file: {db_seed_file.resolve()}"
            )
        if not db_seed_file.suffix == ".py":
            raise typer.BadParameter("Database setup file must be a Python script (.py)")

    if settings.CANVAS_SDK_DB_BACKEND != "sqlite3":
        raise typer.BadParameter(
            "Database backend must be 'sqlite3' for local plugin development. Please unset 'DATABASE_URL' env var."
        )

    if db_seed_file or reset_db:
        _reset_db(seed_file=db_seed_file)

    run_server(plugin_directories)



from datetime import date
from typing import Literal
from uuid import UUID

from pydantic import Field
from pydantic_core import InitErrorDetails

from canvas_sdk.commands.base import _BaseCommand
from canvas_sdk.commands.constants import Coding
from canvas_sdk.v1.data import NoteType, ReasonForVisitSettingCoding


class FollowUpCommand(_BaseCommand):
    """A class for managing a Follow-Up command within a specific note."""

    class Meta:
        key = "followUp"

    structured: bool = False
    requested_date: date | None = None
    note_type_id: UUID | str | None = Field(
        default=None, json_schema_extra={"commands_api_name": "note_type"}
    )
    reason_for_visit: Coding | UUID | str | None = None
    comment: str | None = None

    def _get_error_details(
        self, method: Literal["originate", "edit", "delete", "commit", "enter_in_error"]
    ) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if self.reason_for_visit:
            if self.structured:
                if isinstance(self.reason_for_visit, str | UUID):
                    query = {"id": self.reason_for_visit}
                    error_message = f"ReasonForVisitSettingCoding with id {self.reason_for_visit} does not exist."
                else:
                    query = {
                        "code": self.reason_for_visit["code"],
                        "system": self.reason_for_visit["system"],
                    }
                    error_message = f"ReasonForVisitSettingCoding with code {self.reason_for_visit['code']} and system {self.reason_for_visit['system']} does not exist."

                if not ReasonForVisitSettingCoding.objects.filter(**query).exists():
                    errors.append(
                        self._create_error_detail("value", error_message, self.reason_for_visit)
                    )
            elif not isinstance(self.reason_for_visit, str):
                errors.append(
                    self._create_error_detail(
                        "value",
                        "reason for visit must be a string when structured is False.",
                        self.reason_for_visit,
                    )
                )

        if (
            self.note_type_id
            and not NoteType.objects.filter(
                id=self.note_type_id, is_active=True, is_scheduleable=True
            ).exists()
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    f"NoteType with id {self.note_type_id} does not exist or is not scheduleable.",
                    self.note_type_id,
                )
            )

        return errors


__exports__ = ("FollowUpCommand",)



import factory

from canvas_sdk.v1.data import Facility


class FacilityFactory(factory.django.DjangoModelFactory[Facility]):
    """Factory for creating a Facility."""

    class Meta:
        model = Facility

    name = factory.Faker("company")
    npi_number = factory.Faker("numerify", text="##########")  # 10 digit NPI number
    phone_number = factory.Faker("numerify", text="##########")  # 10 digit phone number
    active = True
    line1 = factory.Faker("street_address")
    line2 = factory.Faker("secondary_address")
    city = factory.Faker("city")
    district = factory.Faker("city_suffix")
    state_code = factory.Faker("state_abbr")
    postal_code = factory.Faker("postcode")



from typing import Any

from canvas_sdk.effects.metadata import BaseMetadata
from canvas_sdk.v1.data import Appointment


class AppointmentsMetadata(BaseMetadata):
    """Effect to upsert an Appointment Metadata record."""

    class Meta:
        effect_type = "APPOINTMENT_METADATA"

    appointment_id: str

    def _get_error_details(self, method: Any) -> list:
        errors = super()._get_error_details(method)

        if not Appointment.objects.filter(id=self.appointment_id).exists():
            errors.append(
                self._create_error_detail(
                    "appointment_id",
                    f"Appointment with id: {self.appointment_id} does not exist.",
                    self.appointment_id,
                )
            )

        return errors


__exports__ = ("AppointmentsMetadata",)



from enum import Enum
from typing import Any

from pydantic import Field

from canvas_sdk.effects.base import EffectType, _BaseEffect


class PatientChartSummaryConfiguration(_BaseEffect):
    """
    An Effect that will decide which sections appear on the patient's chart summary in Canvas.
    """

    class Meta:
        effect_type = EffectType.SHOW_PATIENT_CHART_SUMMARY_SECTIONS

    class Section(Enum):
        SOCIAL_DETERMINANTS = "social_determinants"
        GOALS = "goals"
        CONDITIONS = "conditions"
        MEDICATIONS = "medications"
        ALLERGIES = "allergies"
        CARE_TEAMS = "care_teams"
        VITALS = "vitals"
        IMMUNIZATIONS = "immunizations"
        SURGICAL_HISTORY = "surgical_history"
        FAMILY_HISTORY = "family_history"
        CODING_GAPS = "coding_gaps"

    sections: list[Section] = Field(min_length=1)

    @property
    def values(self) -> dict[str, Any]:
        """The PatientChartSummaryConfiguration's values."""
        return {"sections": [s.value for s in self.sections]}

    @property
    def effect_payload(self) -> dict[str, Any]:
        """The payload of the effect."""
        return {"data": self.values}


__exports__ = ("PatientChartSummaryConfiguration",)



from decimal import Decimal
from typing import TYPE_CHECKING, Self

from django.db import models

from canvas_sdk.v1.data.base import AuditedModel, IdentifiableModel, TimestampedModel
from canvas_sdk.v1.data.common import PersonSex
from canvas_sdk.v1.data.coverage import (
    CoverageRelationshipCode,
    CoverageType,
    TransactorCoverageType,
)
from canvas_sdk.v1.data.fields import ChoiceArrayField
from canvas_sdk.v1.data.utils import quantize

if TYPE_CHECKING:
    from canvas_sdk.v1.data.claim_line_item import ClaimLineItemQuerySet


class InstallmentPlanStatus(models.TextChoices):
    """InstallmentPlanStatus."""

    ACTIVE = "active", "Active"
    COMPLETED = "completed", "Completed"
    CANCELLED = "cancelled", "Cancelled"


class InstallmentPlan(TimestampedModel):
    """InstallmentPlan."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_installmentplan_001"

    creator = models.ForeignKey("v1.CanvasUser", on_delete=models.CASCADE)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.CASCADE, related_name="installment_plans"
    )
    total_amount = models.DecimalField(max_digits=8, decimal_places=2)
    status = models.CharField(choices=InstallmentPlanStatus.choices, max_length=10)
    expected_payoff_date = models.DateField()


class ClaimQueueColumns(models.TextChoices):
    """ClaimQueueColumns."""

    NOTE_TYPE = "NoteType", "Note type"
    CLAIM_ID = "ClaimID", "Claim ID"
    DOS = "DateOfService", "Date of service"
    PATIENT = "Patient", "Patient"
    ACTIVE_INSURANCE = "ActiveInsurance", "Active insurance"
    INSURANCE_BALANCE = "InsuranceBalance", "Insurance balance"
    PATIENT_BALANCE = "PatientBalance", "Patient balance"
    DAYS_IN_QUEUE = "DaysInQueue", "Days in queue"
    PROVIDER = "Provider", "Provider"
    GUARANTOR = "Guarantor", "Guarantor"
    LATEST_REMIT = "LatestRemit", "Latest remit"
    LAST_INVOICED = "LastInvoiced", "Last invoiced"
    SNOOZED_UNTIL = "SnoozedUntil", "Snoozed until"
    LABELS = "Labels", "Labels"


class ClaimQueues(models.IntegerChoices):
    """ClaimQueues."""

    APPOINTMENT = 1, "Appointment"
    NEEDS_CLINICIAN_REVIEW = 2, "NeedsClinicianReview"
    NEEDS_CODING_REVIEW = 3, "NeedsCodingReview"
    QUEUED_FOR_SUBMISSION = 4, "QueuedForSubmission"
    FILED_AWAITING_RESPONSE = 5, "FiledAwaitingResponse"
    REJECTED_NEEDS_REVIEW = 6, "RejectedNeedsReview"
    ADJUDICATED_OPEN_BALANCE = 7, "AdjudicatedOpenBalance"
    PATIENT_BALANCE = 8, "PatientBalance"
    ZERO_BALANCE = 9, "ZeroBalance"
    TRASH = 10, "Trash"


class ClaimQueue(TimestampedModel):
    """ClaimQueue."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_queue_001"

    queue_sort_ordering = models.IntegerField()
    name = models.CharField(max_length=100)
    display_name = models.CharField(max_length=100)
    description = models.CharField(max_length=500)
    show_in_revenue = models.BooleanField(default=True)
    visible_columns = ChoiceArrayField(
        models.CharField(choices=ClaimQueueColumns.choices, max_length=64)
    )


class ClaimPayerOrder(models.TextChoices):
    """ClaimPayerOrder."""

    PRIMARY = "Primary", "Primary"
    SECONDARY = "Secondary", "Secondary"
    TERTIARY = "Tertiary", "Tertiary"
    QUATERNARY = "Quaternary", "Quaternary"
    QUINARY = "Quinary", "Quinary"


class ClaimTypeCode(models.TextChoices):
    """ClaimTypeCode."""

    WORKING_AGED = "12", "Working Aged (Age 65 or older)"
    ESRD = "13", "End-Stage Renal Disease"
    NO_FAULT = "14", "No-fault"
    WORKERS_COMP = "15", "Workers Compensation"
    BLACK_LUNG = "41", "Black Lung"
    VA = "42", "Veterans Administration"
    DISABLED = "43", "Disabled (Under Age 65)"
    OTHER_LIABILITY = "47", "Other Liability Insurance is primary"
    UNNECESSARY = "", "No Typecode necessary"


class ClaimComment(IdentifiableModel, AuditedModel):
    """ClaimComment."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimcomment_001"

    claim = models.ForeignKey("Claim", on_delete=models.CASCADE, related_name="comments")
    comment = models.TextField()


class ClaimCoverageQuerySet(models.QuerySet):
    """ClaimCoverageQuerySet."""

    def active(self) -> Self:
        """Filter active claim coverages."""
        return self.filter(active=True)


class ClaimCoverage(TimestampedModel, IdentifiableModel):
    """A model that represents the link between a claim and a specific insurance coverage."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimcoverage_001"

    objects = ClaimCoverageQuerySet.as_manager()

    claim = models.ForeignKey("Claim", on_delete=models.CASCADE, related_name="coverages")

    coverage = models.ForeignKey(
        "v1.Coverage", related_name="claim_coverages", on_delete=models.PROTECT
    )

    active = models.BooleanField(default=True)
    payer_name = models.CharField(max_length=255, default="", blank=True, db_index=True)
    payer_id = models.CharField(max_length=255, default="", blank=True)
    payer_typecode = models.CharField(
        max_length=2, choices=ClaimTypeCode.choices, default=ClaimTypeCode.UNNECESSARY
    )
    payer_order = models.CharField(
        choices=ClaimPayerOrder.choices, max_length=10, default=ClaimPayerOrder.PRIMARY
    )
    payer_addr1 = models.CharField(max_length=255, default="", blank=True)
    payer_addr2 = models.CharField(max_length=255, default="", blank=True)
    payer_city = models.CharField(max_length=255, default="", blank=True)
    payer_state = models.CharField(max_length=2, default="", blank=True)
    payer_zip = models.CharField(max_length=255, default="", blank=True)
    payer_plan_type = models.CharField(
        choices=CoverageType.choices, max_length=20, default="", blank=True
    )
    coverage_type = models.CharField(
        choices=TransactorCoverageType.choices, max_length=64, null=True, blank=True
    )

    subscriber_employer = models.CharField(max_length=255, default="", blank=True)
    subscriber_group = models.CharField(max_length=255, default="", blank=True)
    subscriber_number = models.CharField(max_length=100, default="", blank=True)
    subscriber_plan = models.CharField(max_length=255, default="", blank=True)
    subscriber_dob = models.CharField(max_length=10, default="0000-00-00")
    subscriber_first_name = models.CharField(max_length=255, default="", blank=True)
    subscriber_last_name = models.CharField(max_length=255, default="", blank=True)
    subscriber_middle_name = models.CharField(max_length=255, default="", blank=True, null=True)
    subscriber_phone = models.CharField(max_length=50, default="", blank=True)
    subscriber_sex = models.CharField(
        max_length=3, choices=PersonSex.choices, default=PersonSex.SEX_BLANK
    )
    subscriber_addr1 = models.CharField(max_length=255, default="", blank=True)
    subscriber_addr2 = models.CharField(max_length=255, default="", blank=True, null=True)
    subscriber_city = models.CharField(max_length=255, default="", blank=True)
    subscriber_state = models.CharField(max_length=2, default="", blank=True)
    subscriber_zip = models.CharField(max_length=255, default="", blank=True)
    subscriber_country = models.CharField(max_length=50, default="", blank=True)
    patient_relationship_to_subscriber = models.CharField(
        max_length=2,
        choices=CoverageRelationshipCode.choices,
        default=CoverageRelationshipCode.SELF,
    )

    pay_to_addr1 = models.CharField(max_length=255, default="", blank=True)
    pay_to_addr2 = models.CharField(max_length=255, default="", blank=True)
    pay_to_city = models.CharField(max_length=255, default="", blank=True)
    pay_to_state = models.CharField(max_length=2, default="", blank=True)
    pay_to_zip = models.CharField(max_length=255, default="", blank=True)

    resubmission_code = models.CharField(max_length=1, default="", blank=True)
    payer_icn = models.CharField(max_length=250, default="", blank=True)


class ClaimProvider(TimestampedModel, IdentifiableModel):
    """ClaimProvider."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimprovider_001"

    claim = models.OneToOneField("v1.Claim", on_delete=models.CASCADE, related_name="provider")
    clia_number = models.CharField(max_length=100, default="", blank=True)

    billing_provider_name = models.CharField(max_length=255, default="", blank=True)
    billing_provider_phone = models.CharField(max_length=15, default="", blank=True)
    billing_provider_addr1 = models.CharField(max_length=255, default="", blank=True)
    billing_provider_addr2 = models.CharField(max_length=255, default="", blank=True)
    billing_provider_city = models.CharField(max_length=255, default="", blank=True)
    billing_provider_state = models.CharField(max_length=2, default="", blank=True)
    billing_provider_zip = models.CharField(max_length=255, default="", blank=True)
    billing_provider_id = models.CharField(max_length=255, default="", blank=True)
    billing_provider_npi = models.CharField(max_length=10, default="0")
    billing_provider_tax_id = models.CharField(max_length=100, default="", blank=True)
    billing_provider_tax_id_type = models.CharField(max_length=1, default="E")
    billing_provider_taxonomy = models.CharField(max_length=100, default="", blank=True)

    provider_id = models.CharField(max_length=255, default="", blank=True)
    provider_first_name = models.CharField(max_length=255, default="", blank=True)
    provider_last_name = models.CharField(max_length=255, default="", blank=True)
    provider_middle_name = models.CharField(max_length=255, default="", blank=True)
    provider_npi = models.CharField(max_length=10, default="0")
    provider_tax_id = models.CharField(max_length=100, default="", blank=True)
    provider_tax_id_type = models.CharField(max_length=1, default="E")
    provider_taxonomy = models.CharField(max_length=100, default="", blank=True)
    provider_ptan_identifier = models.CharField(max_length=50, default="", blank=True)

    referring_provider_id = models.CharField(max_length=255, default="", blank=True)
    referring_provider_first_name = models.CharField(max_length=255, default="", blank=True)
    referring_provider_last_name = models.CharField(max_length=255, default="", blank=True)
    referring_provider_middle_name = models.CharField(max_length=255, default="", blank=True)
    referring_provider_npi = models.CharField(max_length=10, default="0")
    referring_provider_ptan_identifier = models.CharField(max_length=50, default="", blank=True)

    ordering_provider_first_name = models.CharField(max_length=255, default="", blank=True)
    ordering_provider_last_name = models.CharField(max_length=255, default="", blank=True)
    ordering_provider_middle_name = models.CharField(max_length=255, default="", blank=True)
    ordering_provider_npi = models.CharField(max_length=10, default="0")

    facility_id = models.CharField(max_length=255, default="", blank=True)
    facility_name = models.CharField(max_length=255, default="", blank=True)
    facility_npi = models.CharField(max_length=10, default="0")
    facility_addr1 = models.CharField(max_length=255, default="", blank=True)
    facility_addr2 = models.CharField(max_length=255, default="", blank=True)
    facility_city = models.CharField(max_length=255, default="", blank=True)
    facility_state = models.CharField(max_length=2, default="", blank=True)
    facility_zip = models.CharField(max_length=255, default="", blank=True)
    hosp_from_date = models.CharField(max_length=10, default="0000-00-00")
    hosp_to_date = models.CharField(max_length=10, default="0000-00-00")


class ClaimPatient(TimestampedModel):
    """ClaimPatient."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimpatient_001"

    claim = models.OneToOneField("v1.Claim", on_delete=models.CASCADE, related_name="patient")
    photo = models.CharField(max_length=512)
    dob = models.CharField(max_length=10)
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)
    middle_name = models.CharField(max_length=255)
    phone = models.CharField(max_length=50)
    sex = models.CharField(choices=PersonSex.choices, max_length=3)
    ssn = models.CharField(max_length=10)
    addr1 = models.CharField(max_length=255)
    addr2 = models.CharField(max_length=255)
    city = models.CharField(max_length=255)
    state = models.CharField(max_length=2)
    zip = models.CharField(max_length=255)
    country = models.CharField(max_length=50)


class ClaimQueryset(models.QuerySet):
    """ClaimQueryset."""

    def active(self) -> Self:
        """Active claims."""
        return self.exclude(current_queue__queue_sort_ordering=ClaimQueues.TRASH)


class Claim(TimestampedModel, IdentifiableModel):
    """Claim."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claim_001"

    objects = ClaimQueryset.as_manager()

    note = models.ForeignKey("v1.Note", on_delete=models.PROTECT, related_name="claims", null=True)
    installment_plan = models.ForeignKey(
        InstallmentPlan, on_delete=models.SET_NULL, related_name="claims", null=True
    )
    current_queue = models.ForeignKey(ClaimQueue, on_delete=models.PROTECT, related_name="claims")
    latest_invoice = models.ForeignKey("v1.Invoice", on_delete=models.SET_NULL, null=True)
    current_coverage = models.ForeignKey(
        ClaimCoverage, related_name="claims", on_delete=models.SET_NULL, null=True
    )

    accept_assign = models.BooleanField(default=True)
    auto_accident = models.BooleanField(default=False)
    auto_accident_state = models.CharField(max_length=2)
    employment_related = models.BooleanField(default=False)
    other_accident = models.BooleanField(default=False)
    accident_code = models.CharField(max_length=10)
    illness_date = models.DateField(blank=True, null=True)
    remote_batch_id = models.CharField(max_length=100)
    remote_file_id = models.CharField(max_length=100)
    prior_auth = models.CharField(max_length=100)

    narrative = models.CharField(max_length=2500)
    account_number = models.CharField(max_length=255)
    snoozed_until = models.DateField(null=True)

    patient_balance = models.DecimalField(max_digits=8, decimal_places=2, default=0.00)
    aggregate_coverage_balance = models.DecimalField(max_digits=8, decimal_places=2, default=0.00)

    @property
    def total_charges(self) -> Decimal:
        """Total charges for the claim."""
        line_items = self.get_active_claim_line_items()
        if line_items:
            return quantize(sum(line_item.charge for line_item in line_items))
        return Decimal("0.00")

    @property
    def total_paid(self) -> Decimal | int:
        """Total paid amount for the claim."""
        return sum(posting.paid_amount for posting in self.postings.active()) or Decimal("0.00")

    @property
    def total_adjusted(self) -> Decimal | int:
        """Total adjusted amount for the claim."""
        return sum(
            posting.adjusted_and_transferred_amount for posting in self.postings.active()
        ) or Decimal("0.00")

    @property
    def balance(self) -> Decimal:
        """Balance for the claim."""
        return self.aggregate_coverage_balance + self.patient_balance

    @property
    def total_patient_paid(self) -> Decimal | int:
        """Total amount paid by the patient."""
        return sum(posting.paid_amount for posting in self.patient.postings.active()) or Decimal(
            "0.00"
        )

    @property
    def total_payer_paid(self) -> Decimal | int:
        """Total amount paid by the coverages."""
        return sum(
            posting.paid_amount
            for coverage in self.coverages.active()
            for posting in coverage.postings.active()
        ) or Decimal("0.00")

    def get_active_claim_line_items(self) -> "ClaimLineItemQuerySet":
        """Return the active claim line items."""
        return self.line_items.active().exclude_copay_and_unlinked()

    def get_coverage_by_payer_id(
        self, payer_id: str, subscriber_number: str | None = None
    ) -> ClaimCoverage | None:
        """Finds the active coverage associated with a payer_id. Optionally checks if the subscriber_number matches,
        which will choose the correct coverage in the case where a patient has two coverages with the same payer_id.
        """
        base_qs = self.coverages.active().filter(payer_id=payer_id)
        qs = base_qs.filter(subscriber_number=subscriber_number) if subscriber_number else base_qs
        return qs.first() or base_qs.first()


class ClaimLabel(IdentifiableModel):
    """ClaimLabel."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimlabel_001"

    claim = models.ForeignKey("v1.Claim", on_delete=models.PROTECT, related_name="claim_labels")
    label = models.ForeignKey("v1.TaskLabel", on_delete=models.PROTECT, related_name="claim_labels")


class ClaimSubmission(IdentifiableModel):
    """ClaimSubmission."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_claimsubmission_001"

    claim = models.ForeignKey("v1.Claim", on_delete=models.PROTECT, related_name="submissions")
    coverage = models.ForeignKey(
        "v1.ClaimCoverage", on_delete=models.PROTECT, related_name="submissions", null=True
    )
    clearinghouse_claim_id = models.CharField(max_length=255, default="", blank=True, null=True)
    claim_index = models.PositiveIntegerField(default=0)


__exports__ = (
    "Claim",
    "ClaimLabel",
    "ClaimQueue",
    "ClaimComment",
    "ClaimCoverage",
    "ClaimPatient",
    "ClaimPayerOrder",
    "ClaimProvider",
    "ClaimQueues",
    "ClaimQueueColumns",
    "ClaimSubmission",
    "ClaimTypeCode",
    "InstallmentPlan",
    "InstallmentPlanStatus",
)



from django.db import models

from canvas_sdk.v1.data.base import Model


class PayorSpecificCharge(Model):
    """Payor Specific Charge."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_payorspecificcharge_001"

    transactor = models.ForeignKey(
        "v1.Transactor", related_name="specific_charges", on_delete=models.DO_NOTHING
    )
    charge = models.ForeignKey(
        "v1.ChargeDescriptionMaster",
        related_name="transactor_charges",
        on_delete=models.DO_NOTHING,
    )
    charge_amount = models.DecimalField(max_digits=8, decimal_places=2)
    effective_date = models.DateField()
    end_date = models.DateField(null=True)
    part_of_capitated_set = models.BooleanField()


__exports__ = ("PayorSpecificCharge",)



import datetime
import json
from dataclasses import dataclass
from typing import Any

from pydantic_core import InitErrorDetails

from canvas_generated.messages.effects_pb2 import Effect
from canvas_sdk.base import TrackableFieldsModel
from canvas_sdk.effects.metadata import Metadata as PatientMetadata
from canvas_sdk.v1.data import Patient as PatientModel
from canvas_sdk.v1.data import PracticeLocation, Staff
from canvas_sdk.v1.data.common import (
    AddressType,
    AddressUse,
    ContactPointSystem,
    ContactPointUse,
    PersonSex,
)


@dataclass
class PatientContactPoint:
    """A class representing a patient contact point."""

    system: ContactPointSystem
    value: str
    use: ContactPointUse
    rank: int
    has_consent: bool | None = None

    def to_dict(self) -> dict[str, Any]:
        """Convert the contact point to a dictionary."""
        return {
            "system": self.system.value,
            "value": self.value,
            "use": self.use.value,
            "rank": self.rank,
            "has_consent": self.has_consent,
        }


@dataclass
class PatientExternalIdentifier:
    """A class representing a patient external identifier."""

    value: str
    system: str | None = None

    def to_dict(self) -> dict[str, Any]:
        """Convert the external identifier to a dictionary."""
        return {
            "system": self.system,
            "value": self.value,
        }


@dataclass
class PatientPreferredPharmacy:
    """A class representing a preferred pharmacy."""

    ncpdp_id: str
    default: bool = False

    def to_dict(self) -> dict[str, Any]:
        """Convert the preferred pharmacy to a dictionary."""
        return {
            "ncpdp_id": self.ncpdp_id,
            "default": self.default,
        }


@dataclass
class PatientAddress:
    """A class representing a patient address."""

    line1: str
    country: str
    line2: str | None = None
    use: AddressUse = AddressUse.HOME
    type: AddressType = AddressType.BOTH
    city: str | None = None
    district: str | None = None
    state_code: str | None = None
    postal_code: str | None = None
    longitude: float | None = None
    latitude: float | None = None

    def to_dict(self) -> dict[str, Any]:
        """Convert the address to a dictionary."""
        return {
            "line1": self.line1,
            "line2": self.line2,
            "country": self.country,
            "use": self.use.value,
            "type": self.type.value,
            "city": self.city,
            "district": self.district,
            "state_code": self.state_code,
            "postal_code": self.postal_code,
            "longitude": self.longitude,
            "latitude": self.latitude,
        }


class Patient(TrackableFieldsModel):
    """Effect to create a Patient record."""

    class Meta:
        effect_type = "PATIENT"

    patient_id: str | None = None
    first_name: str | None = None
    last_name: str | None = None
    middle_name: str | None = None
    birthdate: datetime.date | None = None
    prefix: str | None = None
    suffix: str | None = None
    sex_at_birth: PersonSex | None = None
    nickname: str | None = None
    social_security_number: str | None = None
    administrative_note: str | None = None
    clinical_note: str | None = None
    default_location_id: str | None = None
    default_provider_id: str | None = None
    previous_names: list[str] | None = None
    contact_points: list[PatientContactPoint] | None = None
    external_identifiers: list[PatientExternalIdentifier] | None = None
    preferred_pharmacies: list[PatientPreferredPharmacy] | None = None
    addresses: list[PatientAddress] | None = None
    metadata: list[PatientMetadata] | None = None

    @property
    def values(self) -> dict[str, Any]:
        """Return the values of the patient as a dictionary."""
        values = super().values

        if self.is_dirty("contact_points"):
            values["contact_points"] = (
                [cp.to_dict() for cp in self.contact_points]
                if self.contact_points is not None
                else None
            )

        if self.is_dirty("external_identifiers"):
            values["external_identifiers"] = (
                [ids.to_dict() for ids in self.external_identifiers]
                if self.external_identifiers is not None
                else None
            )

        if self.is_dirty("addresses"):
            values["addresses"] = (
                [addr.to_dict() for addr in self.addresses] if self.addresses is not None else None
            )

        if self.is_dirty("preferred_pharmacies"):
            values["preferred_pharmacies"] = (
                [pharmacy.to_dict() for pharmacy in self.preferred_pharmacies]
                if self.preferred_pharmacies
                else None
            )

        if self.is_dirty("metadata"):
            values["metadata"] = (
                [md.to_dict() for md in self.metadata] if self.metadata is not None else None
            )

        return values

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        # Validate create-specific requirements
        if method == "create":
            if self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Patient ID should not be set when creating a new patient.",
                        self.patient_id,
                    )
                )

            # first_name and last_name are required for create
            if not self.first_name:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "First name is required when creating a new patient.",
                        self.first_name,
                    )
                )

            if not self.last_name:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Last name is required when creating a new patient.",
                        self.last_name,
                    )
                )

        # Validate update-specific requirements
        if method == "update":
            if not self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Patient ID must be set when updating an existing patient.",
                        self.patient_id,
                    )
                )
            elif not PatientModel.objects.filter(id=self.patient_id).exists():
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Patient with ID {self.patient_id} does not exist.",
                        self.patient_id,
                    )
                )

        if (
            self.default_location_id
            and not PracticeLocation.objects.filter(id=self.default_location_id).exists()
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Practice location with ID {self.default_location_id} does not exist.",
                    self.default_location_id,
                )
            )

        if (
            self.default_provider_id
            and not Staff.objects.filter(id=self.default_provider_id).exists()
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Provider with ID {self.default_provider_id} does not exist.",
                    self.default_provider_id,
                )
            )

        return errors

    def create(self) -> Effect:
        """Create a new Patient."""
        self._validate_before_effect("create")

        return Effect(
            type=f"CREATE_{self.Meta.effect_type}",
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )

    def update(self) -> Effect:
        """Update an existing Patient."""
        self._validate_before_effect("update")

        return Effect(
            type=f"UPDATE_{self.Meta.effect_type}",
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )


__exports__ = (
    "Patient",
    "PatientAddress",
    "PatientContactPoint",
    "PatientExternalIdentifier",
    "PatientMetadata",
    "PatientPreferredPharmacy",
)



from canvas_sdk.clients.llms.libraries.llm_anthropic import LlmAnthropic
from canvas_sdk.clients.llms.libraries.llm_api import LlmApi
from canvas_sdk.clients.llms.libraries.llm_google import LlmGoogle
from canvas_sdk.clients.llms.libraries.llm_openai import LlmOpenai

__all__ = __exports__ = (
    "LlmAnthropic",
    "LlmApi",
    "LlmGoogle",
    "LlmOpenai",
)



from ..value_set import ValueSet


class AceInhibitorOrArbOrArniIngredient(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for medications of angiotensin-converting enzyme (ACE) inhibitor, angiotensin-receptor blocker (ARB), and angiotensin receptor/neprilysin inhibitor (ARNI) ingredients.

    **Data Element Scope:** This value set may use a model element related to Allergy/Intolerance.

    **Inclusion Criteria:** Includes concepts that represent an allergy or intolerance for prescribable angiotensin-converting enzyme (ACE) inhibitor, angiotensin-receptor blocker (ARB), and angiotensin receptor/neprilysin inhibitor (ARNI) ingredients only.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "ACE Inhibitor or ARB or ARNI Ingredient"
    OID = "2.16.840.1.113883.3.526.3.1489"
    DEFINITION_VERSION = "20230217"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    RXNORM = {
        "1091643",  # azilsartan
        "1656328",  # sacubitril
        "18867",  # benazepril
        "1998",  # captopril
        "214354",  # candesartan
        "29046",  # lisinopril
        "30131",  # moexipril
        "321064",  # olmesartan
        "35208",  # quinapril
        "35296",  # ramipril
        "3827",  # enalapril
        "38454",  # trandolapril
        "50166",  # fosinopril
        "52175",  # losartan
        "54552",  # perindopril
        "69749",  # valsartan
        "73494",  # telmisartan
        "83515",  # eprosartan
        "83818",  # irbesartan
    }


class BetaBlockerTherapyIngredient(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for medications containing ingredients for beta blocker therapy.

    **Data Element Scope:** This value set may use a model element related to Allergy/Intolerance.

    **Inclusion Criteria:** Includes concepts that represent an allergy or intolerance for prescribable beta blockers.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Beta Blocker Therapy Ingredient"
    OID = "2.16.840.1.113883.3.526.3.1493"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    RXNORM = {
        "10600",  # timolol
        "10824",  # metipranolol
        "1202",  # atenolol
        "149",  # acebutolol
        "1520",  # betaxolol
        "1813",  # levobunolol
        "19484",  # bisoprolol
        "20352",  # carvedilol
        "2116",  # carteolol
        "31555",  # nebivolol
        "49737",  # esmolol
        "6185",  # labetalol
        "6918",  # metoprolol
        "7226",  # nadolol
        "8332",  # pindolol
        "8787",  # propranolol
        "9947",  # sotalol
    }


class StatinAllergen(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of an allergy to statin medications.

    **Data Element Scope:** This value set may use a model element related to Allergy/Intolerance.

    **Inclusion Criteria:** Includes concepts that define an allergy to statin medications.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Statin Allergen"
    OID = "2.16.840.1.113762.1.4.1110.42"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    RXNORM = {
        "301542",  # rosuvastatin
        "36567",  # simvastatin
        "41127",  # fluvastatin
        "42463",  # pravastatin
        "6472",  # lovastatin
        "83367",  # atorvastatin
        "861634",  # pitavastatin
    }

    SNOMEDCT = {
        "372912004",  # Substance with 3-hydroxy-3-methylglutaryl-coenzyme A reductase inhibitor mechanism of action (substance)
        "96302009",  # Product containing 3-hydroxy-3-methylglutaryl-coenzyme A reductase inhibitor (product)
    }


__exports__ = (
    "AceInhibitorOrArbOrArniIngredient",
    "BetaBlockerTherapyIngredient",
    "StatinAllergen",
)



from canvas_sdk.effects import EffectType
from canvas_sdk.effects.validation.base import (
    ValidationError,
    _BaseValidationErrorEffect,
)


class CommandValidationErrorEffect(_BaseValidationErrorEffect):
    """
    Effect to represent command validation errors.

    This effect allows plugins to return validation errors for commands,
    which will be displayed to users in the Canvas UI.

    Example:
        # Add validation errors
        effect = CommandValidationErrorEffect()
        effect.add_error("Narrative is required")
        effect.add_error("Dosage must be a positive number")
        return [effect.apply()]

        # Or initialize with errors
        errors = [
            ValidationError("Narrative is required"),
            ValidationError("This command cannot be submitted yet")
        ]
        effect = CommandValidationErrorEffect(errors=errors)
        return [effect.apply()]

        # Method chaining
        effect = CommandValidationErrorEffect()
        effect.add_error("Error 1").add_error("Error 2")
        return [effect.apply()]
    """

    class Meta:
        effect_type = EffectType.COMMAND_VALIDATION_ERRORS


__all__ = __exports__ = (
    "CommandValidationErrorEffect",
    "ValidationError",
)



from ..value_set import ValueSet


class LevelOfSeverityOfRetinopathyFindings(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of the level of severity of retinopathy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis or Communication.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of mild, moderate, severe, or very severe non-proliferative and proliferative diabetic retinopathy.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS142v10
    """

    VALUE_SET_NAME = "Level of Severity of Retinopathy Findings"
    OID = "2.16.840.1.113883.3.526.3.1283"
    DEFINITION_VERSION = "20210209"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "59276001",  # Proliferative retinopathy due to diabetes mellitus (disorder)
        "312903003",  # Mild nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312904009",  # Moderate nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312905005",  # Severe nonproliferative retinopathy due to diabetes mellitus (disorder)
        "399876000",  # Very severe nonproliferative retinopathy due to diabetes mellitus (disorder)
    }


class MacularEdemaFindingsPresent(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses to identify the presence of macular edema.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of macular edema.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of non-macular edema.

    ** Used in:** CMS142v10
    """

    VALUE_SET_NAME = "Macular Edema Findings Present"
    OID = "2.16.840.1.113883.3.526.3.1320"
    DEFINITION_VERSION = "20210209"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "37231002",  # Macular retinal edema (disorder)
        "193350004",  # Advanced maculopathy due to diabetes mellitus (disorder)
        "193387007",  # Cystoid macular edema (disorder)
        "232020009",  # Disorder of macula due to diabetes mellitus (disorder)
        "312911008",  # Clinically significant macular edema (disorder)
        "312912001",  # Macular edema due to diabetes mellitus (disorder)
        "312920004",  # Postoperative cystoid macular edema (disorder)
        "312921000",  # Autosomal dominant cystoid macular edema (disorder)
        "312922007",  # Uveitis related cystoid macular edema (disorder)
        "314010006",  # Diffuse exudative maculopathy due to diabetes mellitus (disorder)
        "314011005",  # Focal exudative maculopathy due to diabetes mellitus (disorder)
        "314014002",  # Ischemic maculopathy due to diabetes mellitus (disorder)
        "314015001",  # Mixed maculopathy due to diabetes mellitus (disorder)
        "399864000",  # Macular edema not clinically significant due to diabetes mellitus (disorder)
        "399874002",  # High risk proliferative retinopathy with clinically significant macula edema due to diabetes mellitus (disorder)
        "399875001",  # Non-high-risk proliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "420486006",  # Exudative maculopathy due to type 1 diabetes mellitus (disorder)
        "421779007",  # Exudative maculopathy due to type 2 diabetes mellitus (disorder)
        "432789001",  # Noncystoid edema of macula of retina (disorder)
        "769217008",  # Macular edema of right eye due to diabetes mellitus (disorder)
        "769218003",  # Macular edema of left eye due to diabetes mellitus (disorder)
        "769219006",  # Macular edema due to type 1 diabetes mellitus (disorder)
        "769220000",  # Macular edema due to type 2 diabetes mellitus (disorder)
        "770097006",  # Clinically significant macular edema due to diabetes mellitus (disorder)
        "870529009",  # Persistent macular edema due to diabetes mellitus (disorder)
        "871778008",  # Centrally involved macular edema due to diabetes mellitus (disorder)
        "871781003",  # Non centrally involved macular edema due to diabetes mellitus (disorder)
    }


class ConsultantReport(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for consultant reports.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent various consultant reports and notes.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS50v10
    """

    VALUE_SET_NAME = "Consultant Report"
    OID = "2.16.840.1.113883.3.464.1003.121.12.1006"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "11488-4",  # Consult note
        "34099-2",  # Cardiology Consult note
        "34100-8",  # Intensive care unit Consult note
        "34101-6",  # General medicine Outpatient Consult note
        "34102-4",  # Psychiatry Hospital Consult note
        "34103-2",  # Pulmonary Consult note
        "34104-0",  # Hospital Consult note
        "34749-2",  # Anesthesiology Outpatient Consult note
        "34756-7",  # Dentistry Consult note
        "34758-3",  # Dermatology Consult note
        "34760-9",  # Diabetology Consult note
        "34761-7",  # Gastroenterology Consult note
        "34764-1",  # General medicine Consult note
        "34776-5",  # Geriatric medicine Consult note
        "34777-3",  # Obstetrics and Gynecology Consult note
        "34779-9",  # Hematology+Medical oncology Consult note
        "34781-5",  # Infectious disease Consult note
        "34783-1",  # Kinesiotherapy Consult note
        "34785-6",  # Mental health Consult note
        "34788-0",  # Psychiatry Consult note
        "34791-4",  # Psychology Consult note
        "34795-5",  # Nephrology Consult note
        "34797-1",  # Neurology Consult note
        "34798-9",  # Neurological surgery Consult note
        "34800-3",  # Nutrition and dietetics Consult note
        "34803-7",  # Occupational medicine Consult note
        "34805-2",  # Oncology Consult note
        "34807-8",  # Ophthalmology Consult note
        "34810-2",  # Optometry Consult note
        "34812-8",  # Oral and Maxillofacial Surgery Consult note
        "34814-4",  # Orthopaedic surgery Consult note
        "34816-9",  # Otolaryngology Consult note
        "34820-1",  # Pharmacology Consult note
        "34822-7",  # Physical medicine and rehab Consult note
        "34824-3",  # Physical therapy Consult note
        "34826-8",  # Plastic surgery Consult note
        "34828-4",  # Podiatry Consult note
        "34831-8",  # Radiation oncology Consult note
        "34833-4",  # Recreational therapy Consult note
        "34837-5",  # Respiratory therapy Consult note
        "34839-1",  # Rheumatology Consult note
        "34841-7",  # Social worker Consult note
        "34845-8",  # Speech-language pathology+Audiology Consult note
        "34847-4",  # Surgery Consult note
        "34849-0",  # Cardiothoracic surgery Consult note
        "34851-6",  # Urology Consult note
        "34853-2",  # Vascular surgery Consult note
        "34855-7",  # Occupational therapy Consult note
        "34879-7",  # Endocrinology Consult note
        "51845-6",  # Outpatient Consult note
        "51846-4",  # Emergency department Consult note
        "51854-8",  # Long term care facility Consult note
        "64056-5",  # General medicine Medical student Hospital Consult note
        "64068-0",  # Surgery Medical student Hospital Consult note
        "64072-2",  # Critical care medicine Medical student Hospital Consult note
        "64076-3",  # Cardiothoracic surgery Medical student Hospital Consult note
        "64080-5",  # Pulmonary Medical student Hospital Consult note
        "68469-6",  # Pastoral care Hospital Consult note
        "68486-0",  # Cardiology Medical student Hospital Consult note
        "68551-1",  # Dermatology Hospital Consult note
        "68566-9",  # Obstetrics and Gynecology Hospital Consult note
        "68570-1",  # Occupational therapy Hospital Consult note
        "68575-0",  # Ophthalmology Hospital Consult note
        "68586-7",  # Pharmacology Hospital Consult note
        "68590-9",  # Physical therapy Hospital Consult note
        "68597-4",  # Plastic surgery Hospital Consult note
        "68619-6",  # Adolescent medicine Hospital Consult note
        "68633-7",  # Allergy and Immunology Hospital Consult note
        "68639-4",  # Audiology Hospital Consult note
        "68648-5",  # Child and adolescent psychiatry Hospital Consult note
        "68651-9",  # Clinical biochemical genetics Hospital Consult note
        "68661-8",  # Clinical genetics Hospital Consult note
        "68670-9",  # Developmental-behavioral pediatrics Hospital Consult note
        "68681-6",  # Multi-specialty program Hospital Consult note
        "68685-7",  # Neonatal perinatal medicine Hospital Consult note
        "68694-9",  # Neurological surgery Hospital Consult note
        "68705-3",  # Neurology with special qualifications in child neurology Hospital Consult note
        "68716-0",  # Pain medicine Hospital Consult note
        "68727-7",  # Pediatric cardiology Hospital Consult note
        "68746-7",  # Pediatric gastroenterology Hospital Consult note
        "68757-4",  # Pediatric hematology-oncology Hospital Consult note
        "68765-7",  # Pediatric infectious diseases Hospital Consult note
        "68787-1",  # Pediatric pulmonology Hospital Consult note
        "68802-8",  # Pediatric surgery Hospital Consult note
        "68812-7",  # Pediatric urology Hospital Consult note
        "68821-8",  # Pediatrics Hospital Consult note
        "68837-4",  # Primary care Hospital Consult note
        "68846-5",  # Speech-language pathology Hospital Consult note
        "68852-3",  # Transplant surgery Hospital Consult note
        "68864-8",  # Pediatric transplant hepatology Hospital Consult note
        "68869-7",  # Pediatric nephrology Hospital Consult note
        "68874-7",  # Pediatric otolaryngology Hospital Consult note
        "68879-6",  # Pediatric rheumatology Hospital Consult note
        "68892-9",  # Pediatric dermatology Hospital Consult note
        "68897-8",  # Pediatric endocrinology Hospital Consult note
        "72555-6",  # Interventional radiology Consult note
        "73575-3",  # Radiology Consult note
        "75424-2",  # Audiology Consult note
        "75465-5",  # Orthotics prosthetics Consult note
        "77403-4",  # Anesthesiology Consult note
        "77429-9",  # Allergy and Immunology Consult note
        "78250-8",  # Colon and rectal surgery Consult note
        "78251-6",  # Hematology Consult note
        "78252-4",  # Multi-specialty program Consult note
        "78253-2",  # Family medicine Consult note
        "78254-0",  # Clinical genetics Consult note
        "78405-8",  # Neonatal perinatal medicine Consult note
        "78406-6",  # Nurse practitioner Consult note
        "78496-7",  # Critical care medicine Consult note
        "78498-3",  # Cardiopulmonary Consult note
        "78567-5",  # Pain medicine Consult note
        "78568-3",  # Palliative care Consult note
        "78726-7",  # Pediatrics Consult note
        "78732-5",  # Trauma Consult note
        "78738-2",  # Sports medicine Consult note
        "79428-9",  # Rapid response team Consult note
        "80396-5",  # Gynecologic oncology Consult note
        "80575-4",  # Cardiac surgery Consult note
        "80664-6",  # Clinical neurophysiology Consult note
        "80666-1",  # Vascular neurology Consult note
        "80673-7",  # Maternal and fetal medicine Consult note
        "80736-2",  # Blood banking and transfusion medicine Consult note
        "80801-4",  # Surgical oncology Consult note
        "81191-9",  # Chemical pathology Consult note
        "81192-7",  # Clinical pathology Consult note
        "81193-5",  # Medical microbiology - pathology Consult note
        "81196-8",  # Clinical pharmacology Consult note
        "82356-7",  # Obstetrics Midwife Consult note
        "82359-1",  # Reproductive endocrinology and infertility Consult note
        "83570-2",  # Diabetology Nurse Consult note
        "83578-5",  # Hematology+Medical oncology Nurse Consult note
        "83609-8",  # Psychiatry Nurse Consult note
        "83621-3",  # Geriatric medicine Nurse Consult note
        "83653-6",  # Pain medicine Team Consult note
        "83685-8",  # Hematology+Medical oncology Attending Consult note
        "83720-3",  # Physical medicine and rehab Long term care facility Consult note
        "83722-9",  # Physical medicine and rehabilitation Nurse Consult note
        "83868-0",  # Preventive medicine Consult note
        "83873-0",  # Primary care Consult note
        "83888-8",  # Psychiatry Long term care facility Consult note
        "83909-2",  # Public health Consult note
        "83912-6",  # Pulmonary Patient's home Consult note
        "83926-6",  # Recreational therapy Long term care facility Consult note
        "83931-6",  # Research Consult note
        "83941-5",  # Respiratory therapy Patient's home Consult note
        "83960-5",  # Social worker Patient's home Consult note
        "83967-0",  # Social worker Long term care facility Consult note
        "83984-5",  # Speech-language pathology Consult note
        "83992-8",  # Spinal cord injury medicine Consult note
        "83996-9",  # Spinal cord injury medicine Patient's home Consult note
        "84035-5",  # Transplant surgery Consult note
        "84071-0",  # Chiropractic medicine Consult note
        "84115-5",  # Geriatric medicine Long term care facility Consult note
        "84126-2",  # Surgery of the hand Consult note
        "84131-2",  # Primary care Patient's home Consult note
        "84142-9",  # Hematology+Medical oncology Hospital Consult note
        "84145-2",  # Hematology+Medical oncology Outpatient Consult note
        "84152-8",  # Hepatology Consult note
        "84173-4",  # Interventional cardiology Consult note
        "84190-8",  # Medical toxicology Consult note
        "84213-8",  # Mental health Team Consult note
        "84231-0",  # Neurology Telehealth Consult note
        "84241-9",  # Nurse Consult note
        "84280-7",  # Nutrition and dietetics Patient's home Consult note
        "84292-2",  # Nutrition and dietetics Team Consult note
        "84303-7",  # Wound care management Consult note
        "84312-8",  # Vocational rehabilitation Consult note
        "84324-3",  # Physical therapy Long term care facility Consult note
        "84349-0",  # Pastoral care Consult note
        "84352-4",  # Palliative care Team Consult note
        "84358-1",  # Palliative care Patient's home Consult note
        "84394-6",  # Occupational therapy Patient's home Consult note
        "84398-7",  # Occupational therapy Long term care facility Consult note
        "85174-1",  # Custodial care facility Consult note
        "85208-7",  # Telehealth Consult note
        "85222-8",  # Case manager Consult note
        "85232-7",  # Tumor board Consult note
        "85237-6",  # Acupuncture Consult note
        "85238-4",  # Internal medicine Consult note
        "85517-1",  # Bariatric surgery Consult note
        "85519-7",  # Child and adolescent psychology Consult note
        "85866-2",  # Sleep medicine Consult note
        "85871-2",  # Womens health Consult note
        "85882-9",  # Ophthalmology Teleimaging Consult note
        "85884-5",  # Epilepsy Consult note
        "85886-0",  # Clinical cardiac electrophysiology Consult note
        "85890-2",  # Neuropsychology Consult note
        "85899-3",  # Community health care Consult note
        "86451-2",  # Ethics Consult note
        "87233-3",  # Dialysis Consult note
        "87254-9",  # Addiction medicine Consult note
        "87627-6",  # Brain injury Consult note
        "88351-2",  # Polytrauma Consult note
        "88640-8",  # Community health care Adult day care center Consult note
        "88644-0",  # Outpatient hospital Consult note
        "89031-9",  # Hematology Telehealth Consult note
        "89032-7",  # Nephrology Telehealth Consult note
        "89033-5",  # Urology Telehealth Consult note
        "89216-6",  # Gynecology Consult note
        "89227-3",  # Obstetrics Consult note
        "89446-9",  # Obstetrics Hospital Consult note
        "89447-7",  # Gynecology Hospital Consult note
        "89551-6",  # Environmental health Consult note
        "90006-8",  # Pharmacogenomics Consult note
        "90012-6",  # Burn management Consult note
        "90343-5",  # Nuclear medicine Consult note
        "90354-2",  # Obesity medicine Consult note
        "90709-7",  # Adolescent medicine Consult note
        "90710-5",  # Developmental-behavioral pediatrics Consult note
        "90712-1",  # Sleep medicine Telehealth Consult note
        "90714-7",  # Pain medicine Telehealth Consult note
        "90715-4",  # Palliative care Telehealth Consult note
        "90717-0",  # Heart failure+Transplant cardiology Consult note
        "90771-7",  # Heart failure Consult note
        "91986-0",  # Integrative medicine Consult note
        "92910-9",  # Spinal surgery Consult note
        "92912-5",  # Eating disorders Consult note
        "92915-8",  # Immunology Consult note
        "93024-8",  # Pharmacist Consult note
        "93413-3",  # Mechanical circulatory support Consultation note
        "93955-3",  # Wound, Ostomy, and Continence Care Consult note
    }
    SNOMEDCT = {
        "371530004",  # Clinical consultation report (record artifact)
        "371531000",  # Report of clinical encounter (record artifact)
        "371545006",  # Confirmatory consultation report (record artifact)
    }


__exports__ = (
    "ConsultantReport",
    "LevelOfSeverityOfRetinopathyFindings",
    "MacularEdemaFindingsPresent",
)



import factory

from canvas_sdk.v1.data import (
    Claim,
    ClaimComment,
    ClaimCoverage,
    ClaimLabel,
    ClaimProvider,
    ClaimQueue,
    ClaimSubmission,
)


class ClaimCoverageFactory(factory.django.DjangoModelFactory[ClaimCoverage]):
    """Factory for creating ClaimCoverage."""

    class Meta:
        model = ClaimCoverage

    claim = factory.SubFactory("canvas_sdk.test_utils.factories.claim.ClaimFactory")
    coverage = factory.SubFactory("canvas_sdk.test_utils.factories.CoverageFactory")


class ClaimQueueFactory(factory.django.DjangoModelFactory[ClaimQueue]):
    """Factory for creating ClaimQueue."""

    class Meta:
        model = ClaimQueue
        django_get_or_create = ("queue_sort_ordering",)

    queue_sort_ordering = 5
    name = "Claim: Filed"


class ClaimFactory(factory.django.DjangoModelFactory[Claim]):
    """Factory for creating Claim."""

    class Meta:
        model = Claim

    note = factory.SubFactory("canvas_sdk.test_utils.factories.NoteFactory")
    current_queue = factory.SubFactory(ClaimQueueFactory)


class ClaimCommentFactory(factory.django.DjangoModelFactory[ClaimComment]):
    """Factory for creating ClaimComment."""

    class Meta:
        model = ClaimComment

    claim = factory.SubFactory(ClaimFactory)
    comment = "Need to message Dr. House"


class ClaimProviderFactory(factory.django.DjangoModelFactory[ClaimProvider]):
    """Factory for creating ClaimProvider."""

    class Meta:
        model = ClaimProvider

    claim = factory.SubFactory(ClaimFactory)


class ClaimLabelFactory(factory.django.DjangoModelFactory[ClaimLabel]):
    """Factory for creating ClaimLabel."""

    class Meta:
        model = ClaimLabel

    claim = factory.SubFactory(ClaimFactory)
    label = factory.SubFactory("canvas_sdk.test_utils.factories.TaskLabelFactory")


class ClaimSubmissionFactory(factory.django.DjangoModelFactory[ClaimSubmission]):
    """Factory for creating ClaimSubmission."""

    class Meta:
        model = ClaimSubmission

    claim = factory.SubFactory(ClaimFactory)
    clearinghouse_claim_id = "123456"



import factory

from canvas_sdk.v1.data import Organization, OrganizationAddress, OrganizationContactPoint
from canvas_sdk.v1.data.common import ContactPointSystem, ContactPointUse


class OrganizationFactory(factory.django.DjangoModelFactory[Organization]):
    """Factory for creating Organization."""

    class Meta:
        model = Organization

    short_name = "Canvas Medical"
    full_name = "Canvas Medical, Inc."
    logo_url = "https://s3-us-west-2.amazonaws.com/canvas-pcp-assets/canvas_logo_large.png"
    background_image_url = "https://s3-us-west-2.amazonaws.com/canvas-pcp-assets/santa-cruz-low.jpg"
    background_gradient = (
        "linear-gradient(to top, rgb(231, 201, 154) 0%, rgb(220, 210, 192) 29%, "
        "rgb(156, 175, 191) 90%, rgb(147, 166, 189) 100%)"
    )
    tax_id = "123456"


class OrganizationAddressFactory(factory.django.DjangoModelFactory[OrganizationAddress]):
    """Factory for creating OrganizationAddress."""

    class Meta:
        model = OrganizationAddress

    organization = factory.SubFactory(OrganizationFactory)
    line1 = "456 Organization Ave"
    line2 = "#2"
    city = "San Francisco"
    district = "Sunset"
    state_code = "CA"
    postal_code = "94112"
    country = "USA"


class OrganizationContactPointFactory(factory.django.DjangoModelFactory[OrganizationContactPoint]):
    """Factory for creating OrganizationContactPoint."""

    class Meta:
        model = OrganizationContactPoint

    organization = factory.SubFactory(OrganizationFactory)
    system = ContactPointSystem.PHONE
    value = "8003701416"
    use = ContactPointUse.WORK
    rank = 1



from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.db.models import TextChoices

from canvas_sdk.v1.data.base import (
    IdentifiableModel,
    TimestampedModel,
)
from canvas_sdk.v1.data.coding import Coding


class MedicationHistoryMedication(TimestampedModel, IdentifiableModel):
    """MedicationHistoryMedication."""

    class Meta:
        db_table = "canvas_sdk_data_api_medicationhistorymedication_001"

    patient = models.ForeignKey(
        "v1.Patient",
        on_delete=models.DO_NOTHING,
        related_name="medication_history_medications",
        null=True,
    )

    drug_description = models.TextField(blank=True, default="")

    strength_value = models.CharField(max_length=255, blank=True, default="")
    strength_form = models.CharField(max_length=255, blank=True, default="")
    strength_unit_of_measure = models.CharField(max_length=255, blank=True, default="")

    quantity = models.FloatField(null=True)
    quantity_unit_of_measure = models.CharField(max_length=255, blank=True, default="")
    quantity_code_list_qualifier = models.CharField(max_length=255, blank=True, default="")

    days_supply = models.IntegerField(null=True)

    last_fill_date = models.DateTimeField(db_index=True)
    written_date = models.DateTimeField(blank=True, null=True)

    other_date = models.DateTimeField(blank=True, null=True)
    other_date_qualifier = models.CharField(max_length=255, blank=True, default="")

    substitutions = models.BooleanField(null=True)

    refills_remaining = models.IntegerField(null=True)

    diagnosis_code = models.CharField(max_length=255, blank=True, default="")
    diagnosis_qualifier = models.CharField(max_length=255, blank=True, default="")
    diagnosis_description = models.CharField(max_length=255, blank=True, default="")

    secondary_diagnosis_code = models.CharField(max_length=255, blank=True, default="")
    secondary_diagnosis_qualifier = models.CharField(max_length=255, blank=True, default="")
    secondary_diagnosis_description = models.CharField(max_length=255, blank=True, default="")

    dea_schedule = models.CharField(max_length=255, blank=True, default="")

    potency_unit_code = models.CharField(max_length=20, blank=True, default="")
    etc_path_id = ArrayField(base_field=models.IntegerField(), null=True)
    etc_path_name = ArrayField(base_field=models.CharField(max_length=255), null=True)

    fill_number = models.IntegerField(blank=True, null=True)

    prescriber_order_number = models.CharField(max_length=255, blank=True, default="")

    source_description = models.CharField(max_length=255, blank=True, default="")
    source_qualifier = models.CharField(max_length=255, blank=True, default="")
    source_payer_id = models.CharField(max_length=255, blank=True, default="")
    source_type = models.CharField(max_length=255, blank=True, default="")

    note = models.TextField(blank=True, default="")
    sig = models.TextField(blank=True, default="")

    prior_authorization_status = models.CharField(max_length=255, blank=True, default="")
    prior_authorization = models.CharField(max_length=255, blank=True, default="")

    pharmacy_name = models.CharField(max_length=255, blank=True, default="")
    pharmacy_ncpdp_id = models.CharField(max_length=255, blank=True, default="")
    pharmacy_npi = models.CharField(max_length=255, blank=True, default="")

    prescriber_business_name = models.CharField(max_length=255, blank=True, default="")
    prescriber_first_name = models.CharField(max_length=255, blank=True, default="")
    prescriber_last_name = models.CharField(max_length=255, blank=True, default="")
    prescriber_npi = models.CharField(max_length=255, blank=True, default="")
    prescriber_dea_number = models.CharField(max_length=255, blank=True, default="")


class MedicationHistoryMedicationCoding(Coding):
    """MedicationHistoryMedicationCoding."""

    class Meta:
        db_table = "canvas_sdk_data_api_medicationhistorymedicationcoding_001"

    medication = models.ForeignKey(
        MedicationHistoryMedication, on_delete=models.CASCADE, related_name="codings"
    )


class MedicationHistoryResponseStatus(TextChoices):
    """MedicationHistoryResponseStatus."""

    STATUS_APPROVED = "approved"
    STATUS_DENIED = "denied"


class MedicationHistoryResponse(TimestampedModel, IdentifiableModel):
    """MedicationHistoryResponse."""

    class Meta:
        db_table = "canvas_sdk_data_api_medicationhistoryresponse_001"

    patient = models.ForeignKey(
        "v1.Patient",
        on_delete=models.CASCADE,
        related_name="medication_history_responses",
    )
    staff = models.ForeignKey(
        "v1.Staff",
        on_delete=models.CASCADE,
        related_name="medication_history_responses",
        null=True,
    )

    message_id = models.CharField(max_length=35, blank=True, default="", db_index=True)
    related_to_message_id = models.CharField(max_length=35, blank=True, default="", db_index=True)

    status = models.CharField(choices=MedicationHistoryResponseStatus.choices, max_length=20)

    reason = models.TextField(blank=True, default="")
    reason_code = models.CharField(max_length=2, blank=True, default="")

    note = models.TextField(blank=True, default="")

    start_date = models.DateField()
    end_date = models.DateField()


__exports__ = (
    "MedicationHistoryMedication",
    "MedicationHistoryMedicationCoding",
    "MedicationHistoryResponseStatus",
    "MedicationHistoryResponse",
)



import importlib
import json
import pkgutil
from collections.abc import Iterable
from pathlib import Path
from typing import Any

CANVAS_TOP_LEVEL_MODULES = (
    "canvas_sdk.caching",
    "canvas_sdk.clients",
    "canvas_sdk.commands",
    "canvas_sdk.effects",
    "canvas_sdk.events",
    "canvas_sdk.handlers",
    "canvas_sdk.protocols",
    "canvas_sdk.questionnaires",
    "canvas_sdk.templates",
    "canvas_sdk.utils",
    "canvas_sdk.v1",
    "canvas_sdk.value_set",
    "canvas_sdk.views",
    "logger",
)


def find_submodules(starting_modules: Iterable[str]) -> list[str]:
    """
    Given a list of modules, return a list of those modules and their submodules.
    """
    submodules = set(starting_modules)

    for module_path in starting_modules:
        try:
            module = importlib.import_module(module_path)

            if not hasattr(module, "__path__"):
                continue

            for _, name, _ in pkgutil.walk_packages(module.__path__, prefix=module.__name__ + "."):
                submodules.add(name)

        except Exception as e:
            print(f"could not import {module_path}: {e}")

    return sorted(submodules)


def main() -> None:
    """
    Generate a JSON file of the allowed canvas_sdk imports.
    """
    print("Generating allowed canavs_sdk imports...")

    CANVAS_SUBMODULE_NAMES = [
        found_module
        for found_module in find_submodules(CANVAS_TOP_LEVEL_MODULES)
        # tests are excluded from the built and distributed module in pyproject.toml
        if "tests" not in found_module and "test_" not in found_module
    ]

    CANVAS_MODULES: dict[str, set[str]] = {}

    for module_name in CANVAS_SUBMODULE_NAMES:
        module = importlib.import_module(module_name)

        exports = getattr(module, "__exports__", None)

        if not exports:
            continue

        if module_name not in CANVAS_MODULES:
            CANVAS_MODULES[module_name] = set()

        CANVAS_MODULES[module_name].update(exports)

    # In use by a current plugin...
    CANVAS_MODULES["canvas_sdk.commands"].add("*")

    def default(o: Any) -> Any:
        if isinstance(o, set):
            return sorted(o)

        raise TypeError(f"Object of type {type(o).__name__} is not JSON serializable")

    allowed_module_imports_path = Path(__file__).parent / "allowed-module-imports.json"

    allowed_module_imports_path.write_text(
        json.dumps(
            CANVAS_MODULES,
            default=default,
            indent=2,
            sort_keys=True,
        )
    )


if __name__ == "__main__":
    main()



from datetime import date
from enum import StrEnum
from typing import NotRequired

from typing_extensions import TypedDict

from canvas_sdk.commands.base import _BaseCommand as BaseCommand


class AssigneeType(StrEnum):
    """The type of assigner for a Task command."""

    ROLE = "role"
    TEAM = "team"
    UNASSIGNED = "unassigned"
    STAFF = "staff"


class TaskAssigner(TypedDict):
    """A class for managing an assign for a Task command."""

    to: AssigneeType
    id: NotRequired[int]


class TaskCommand(BaseCommand):
    """A class for managing a Task command within a specific note."""

    class Meta:
        key = "task"

    title: str = ""
    assign_to: TaskAssigner | None = None
    due_date: date | None = None
    comment: str | None = None
    labels: list[str] | None = None
    linked_items_urns: list[str] | None = None


__exports__ = (
    "AssigneeType",
    "TaskAssigner",
    "TaskCommand",
)



from .surescripts_messages import (
    SendSurescriptsBenefitsRequestEffect,
    SendSurescriptsEligibilityRequestEffect,
    SendSurescriptsMedicationHistoryRequestEffect,
)

__all__ = __exports__ = (
    "SendSurescriptsBenefitsRequestEffect",
    "SendSurescriptsEligibilityRequestEffect",
    "SendSurescriptsMedicationHistoryRequestEffect",
)
